@@ -1,7 +1,7 @@
 //ctl.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of libroar a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -41,7 +41,8 @@ int roar_get_clientid  (struct roar_connection * con) {
  memset(&mes, 0, sizeof(struct roar_message)); // make valgrind happy!
 
  mes.cmd     = ROAR_CMD_WHOAMI;
- mes.datalen = 0;
+ mes.stream  = -1;
+ mes.datalen =  0;
 
  if ( roar_req(con, &mes, NULL) == -1 )
   return -1;
@@ -55,13 +56,24 @@ int roar_get_clientid  (struct roar_connection * con) {
  return mes.data[0];
 }
 
-int roar_server_oinfo   (struct roar_connection * con, struct roar_stream * sa) {
+int roar_server_oinfo   (struct roar_connection * con, struct roar_stream * s) {
+ return roar_server_oinfo2(con, s, -1);
+}
+
+int roar_server_oinfo2  (struct roar_connection * con, struct roar_stream * s, int dir) {
  struct roar_message mes;
 
  memset(&mes, 0, sizeof(struct roar_message)); // make valgrind happy!
 
  mes.cmd     = ROAR_CMD_SERVER_OINFO;
- mes.datalen = 0;
+
+ if ( dir == -1 ) {
+  mes.datalen = 0;
+ } else {
+  mes.datalen = 2;
+  mes.data[0] = 0;
+  mes.data[1] = dir;
+ }
 
  if ( roar_req(con, &mes, NULL) == -1 )
   return -1;
@@ -69,7 +81,7 @@ int roar_server_oinfo   (struct roar_connection * con, struct roar_stream * sa)
  if ( mes.cmd != ROAR_CMD_OK )
   return -1;
 
- if ( roar_stream_m2s(sa, &mes) == -1 )
+ if ( roar_stream_m2s(s, &mes) == -1 )
   return -1;
 
  return 0;
@@ -81,7 +93,8 @@ int roar_get_standby   (struct roar_connection * con) {
  memset(&mes, 0, sizeof(struct roar_message)); // make valgrind happy!
 
  mes.cmd = ROAR_CMD_GET_STANDBY;
- mes.datalen = 0;
+ mes.stream  = -1;
+ mes.datalen =  0;
 
  if ( roar_req(con, &mes, NULL) == -1 )
   return -1;
@@ -98,7 +111,8 @@ int roar_set_standby   (struct roar_connection * con, int state) {
  memset(&mes, 0, sizeof(struct roar_message)); // make valgrind happy!
 
  mes.cmd = ROAR_CMD_SET_STANDBY;
- mes.datalen = 2;
+ mes.stream  = -1;
+ mes.datalen =  2;
 
  *((uint16_t*)mes.data) = ROAR_HOST2NET16((unsigned) state);
 
@@ -121,7 +135,8 @@ int roar_terminate (struct roar_connection * con, int terminate) {
  memset(&mes, 0, sizeof(struct roar_message)); // make valgrind happy!
 
  mes.cmd     = ROAR_CMD_EXIT;
- mes.datalen = 1;
+ mes.stream  = -1;
+ mes.datalen =  1;
  mes.data[0] = terminate;
 
  if ( roar_req(con, &mes, NULL) == -1 )
@@ -133,12 +148,12 @@ int roar_terminate (struct roar_connection * con, int terminate) {
  return 0;
 }
 
-int roar_list         (struct roar_connection * con, int * items,   int max, int cmd) {
+int roar_list_filtered(struct roar_connection * con, int * items,   int max, int cmd, unsigned char filter, unsigned char cmp, uint32_t id) {
  struct roar_message m;
 
  memset(&m, 0, sizeof(struct roar_message)); // make valgrind happy!
 
- roar_ctl_f2m_any(&m);
+ roar_ctl_f2m(&m, filter, cmp, id);
  m.cmd = cmd;
 
  if ( roar_req(con, &m, NULL) == -1 )
@@ -147,13 +162,18 @@ int roar_list         (struct roar_connection * con, int * items,   int max, int
  return roar_ctl_m2ia(&m, items, max);
 }
 
+int roar_list         (struct roar_connection * con, int * items,   int max, int cmd) {
+ return roar_list_filtered(con, items, max, cmd, ROAR_CTL_FILTER_ANY, ROAR_CTL_CMP_ANY, ROAR_CTL_FILTER_ANY);
+}
+
 int roar_get_client   (struct roar_connection * con, struct roar_client * client, int id) {
  struct roar_message m;
 
  memset(&m, 0, sizeof(struct roar_message)); // make valgrind happy!
 
  m.cmd     = ROAR_CMD_GET_CLIENT;
- m.datalen = 1;
+ m.stream  = -1;
+ m.datalen =  1;
  m.data[0] = id;
 
  ROAR_DBG("roar_get_client(*): id = %i", id);
@@ -178,10 +198,10 @@ int roar_get_stream   (struct roar_connection * con, struct roar_stream * stream
  m.datalen = 1;
  m.data[0] = id;
 
- roar_errno = ROAR_ERROR_UNKNOWN;
+ roar_err_set(ROAR_ERROR_UNKNOWN);
 
  if ( roar_req(con, &m, NULL) == -1 ) {
-  roar_errno = ROAR_ERROR_PROTO;
+  roar_err_set(ROAR_ERROR_PROTO);
   return -1;
  }
 
@@ -212,6 +232,10 @@ int roar_kick         (struct roar_connection * con, int type, int id) {
 }
 
 int roar_set_vol      (struct roar_connection * con, int id, struct roar_mixer_settings * mixer, int channels) {
+ return roar_set_vol2(con, id, mixer, channels, ROAR_SET_VOL_ALL);
+}
+
+int roar_set_vol2     (struct roar_connection * con, int id, struct roar_mixer_settings * mixer, int   channels, int mode) {
  struct roar_message m;
  uint16_t * info = (uint16_t *) m.data;
  int i;
@@ -223,7 +247,7 @@ int roar_set_vol      (struct roar_connection * con, int id, struct roar_mixer_s
  m.stream  = id;
  info[0] = ROAR_HOST2NET16(1);
  info[1] = ROAR_HOST2NET16(mixer->scale);
- info[2] = ROAR_HOST2NET16(ROAR_SET_VOL_ALL);
+ info[2] = ROAR_HOST2NET16(mode);
 
  for (i = 0; i < channels; i++)
   info[i+3] = ROAR_HOST2NET16(mixer->mixer[i]);
@@ -286,33 +310,61 @@ int roar_ctl_f2m      (struct roar_message * m, unsigned char   filter, unsigned
  m->data[0] = 0;
  m->data[1] = filter;
  m->data[2] = cmp;
- *((uint32_t*)&(m->data[3])) = ROAR_HOST2NET32(id);
+ m->data[3] = (id & 0xFF000000LU) >> 24;
+ m->data[4] = (id & 0x00FF0000LU) >> 16;
+ m->data[5] = (id & 0x0000FF00LU) >>  8;
+ m->data[6] = (id & 0x000000FFLU) >>  0;
 
  return 0;
 }
 int roar_ctl_m2f      (struct roar_message * m, unsigned char * filter, unsigned char * cmp, uint32_t * id) {
+ register uint32_t idreg;
 
  if ( m->datalen != 7 )
   return -1;
 
  if ( m->data[0] != 0 ) {
-  ROAR_ERR("roar_ctl_m2f(*): version %i not supported!", m->data[0]);
+  ROAR_DBG("roar_ctl_m2f(*): version %i not supported!", (int)m->data[0]);
+  roar_err_set(ROAR_ERROR_NSVERSION);
   return -1;
  }
 
  *filter = m->data[1];
  *cmp    = m->data[2];
 
- *id = ROAR_NET2HOST32(*((uint32_t*)&(m->data[3])));
+ idreg   = ((uint32_t)(unsigned char)m->data[3]) << 24;
+ idreg  |= ((uint32_t)(unsigned char)m->data[4]) << 16;
+ idreg  |= ((uint32_t)(unsigned char)m->data[5]) <<  8;
+ idreg  |= ((uint32_t)(unsigned char)m->data[6]) <<  0;
+ *id     = idreg;
 
  return 0;
 }
 
+int roar_filter_match (const unsigned cmp, const uint32_t a, const uint32_t b) {
+ switch (cmp) {
+  case ROAR_CTL_CMP_ANY:
+    return 1;
+   break;
+  case ROAR_CTL_CMP_EQ:
+    return a == b;
+   break;
+  case ROAR_CTL_CMP_NE:
+    return a != b;
+   break;
+  default:
+    roar_err_set(ROAR_ERROR_NOTSUP);
+    return -1;
+ }
+}
+
 int roar_ctl_ia2m     (struct roar_message * m, int * data, int len) {
  int i;
 
- if ( len > LIBROAR_BUFFER_MSGDATA )
+ if ( len > LIBROAR_BUFFER_MSGDATA ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
  m->datalen = len;
 
@@ -324,8 +376,10 @@ int roar_ctl_ia2m     (struct roar_message * m, int * data, int len) {
 int roar_ctl_m2ia     (struct roar_message * m, int * data, int len) {
  int i;
 
- if ( m->datalen > len )
+ if ( m->datalen > len ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
  for (i = 0; i < m->datalen; i++)
   data[i] = m->data[i];
@@ -339,7 +393,8 @@ int roar_ctl_c2m      (struct roar_message * m, struct roar_client * c) {
  int i;
  int max_len;
  uint32_t pid;
- size_t len_rest;
+ signed long int len_rest;
+ size_t nnode_len;
 
  if ( c == NULL )
   return -1;
@@ -388,9 +443,14 @@ int roar_ctl_c2m      (struct roar_message * m, struct roar_client * c) {
  memcpy(&(m->data[cur]), &pid, 4);
  cur += 4;
 
- len_rest = sizeof(m->data) - cur;
- if ( roar_nnode_to_blob(&(c->nnode), &(m->data[cur]), &len_rest) == 0 ) {
-  cur += len_rest;
+ len_rest = (long int)sizeof(m->data) - (long int)cur;
+ if ( len_rest < 0 ) {
+  roar_panic(ROAR_FATAL_ERROR_MEMORY_CORRUPTION, NULL);
+ }
+
+ nnode_len = len_rest;
+ if ( roar_nnode_to_blob(&(c->nnode), &(m->data[cur]), &nnode_len) == 0 ) {
+  cur += nnode_len;
  }
 
  m->datalen = cur;
@@ -404,21 +464,28 @@ int roar_ctl_m2c      (struct roar_message * m, struct roar_client * c) {
  uint32_t pid;
  size_t len;
 
- if ( m == NULL || c == NULL )
+ if ( m == NULL || c == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( m->datalen == 0 )
+ if ( m->datalen == 0 ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
  ROAR_DBG("roar_ctl_m2c(*): got data!, len = %i", m->datalen);
 
  if ( m->data[0] != 0 ) {
   ROAR_DBG("roar_ctl_m2c(*): wrong version!");
+  roar_err_set(ROAR_ERROR_NSVERSION);
   return -1;
  }
 
- if ( m->datalen < 3 )
+ if ( m->datalen < 3 ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
  ROAR_DBG("roar_ctl_m2c(*): have usable data!");
 
@@ -427,6 +494,11 @@ int roar_ctl_m2c      (struct roar_message * m, struct roar_client * c) {
  for (i = 0; i < ROAR_CLIENTS_MAX_STREAMS_PER_CLIENT; i++)
   c->streams[i] = -1;
 
+ if ( m->data[2] < 0 || m->data[2] > ROAR_CLIENTS_MAX_STREAMS_PER_CLIENT ) {
+  roar_err_set(ROAR_ERROR_TOOMANYARGS);
+  return -1;
+ }
+
  for (i = 0; i < m->data[2]; i++)
   c->streams[i] = m->data[3+i];
 
@@ -481,53 +553,57 @@ int roar_ctl_m2c      (struct roar_message * m, struct roar_client * c) {
  return 0;
 }
 
-int    roar_str2proto (char * proto) {
- if ( !strcasecmp(proto, "roar") ) {
-  return ROAR_PROTO_ROARAUDIO;
- } else if ( !strcasecmp(proto, "roaraudio") ) {
-  return ROAR_PROTO_ROARAUDIO;
- } else if ( !strcasecmp(proto, "esd") ) {
-  return ROAR_PROTO_ESOUND;
- } else if ( !strcasecmp(proto, "esound") ) {
-  return ROAR_PROTO_ESOUND;
- } else if ( !strcasecmp(proto, "auto") ) {
-  return ROAR_PROTO_AUTO;
- } else if ( !strcasecmp(proto, "(auto)") ) {
-  return ROAR_PROTO_AUTO;
- } else if ( !strcasecmp(proto, "http") ) {
-  return ROAR_PROTO_HTTP;
- } else if ( !strcasecmp(proto, "gopher") ) {
-  return ROAR_PROTO_GOPHER;
- } else if ( !strcasecmp(proto, "icy") ) {
-  return ROAR_PROTO_ICY;
- } else if ( !strcasecmp(proto, "simple") ) {
-  return ROAR_PROTO_SIMPLE;
- } else if ( !strcasecmp(proto, "rsound") ) {
-  return ROAR_PROTO_RSOUND;
- } else if ( !strcasecmp(proto, "rplay") ) {
-  return ROAR_PROTO_RPLAY;
- }
+static struct {
+ int id;
+ const char * name;
+} _proto[] = {
+ {ROAR_PROTO_NONE,        "none"},
+ {ROAR_PROTO_ROARAUDIO,   "RoarAudio"},
+ {ROAR_PROTO_ESOUND,      "EsounD"},
+ {ROAR_PROTO_ESOUND,      "ESD"}, // alias
+ {ROAR_PROTO_AUTO,        "(auto)"},
+ {ROAR_PROTO_HTTP,        "http"},
+ {ROAR_PROTO_GOPHER,      "gopher"},
+ {ROAR_PROTO_ICY,         "ICY"},
+ {ROAR_PROTO_SIMPLE,      "Simple"},
+ {ROAR_PROTO_RSOUND,      "RSound"},
+ {ROAR_PROTO_RPLAY,       "RPlay"},
+ {ROAR_PROTO_IRC,         "IRC"},
+ {ROAR_PROTO_DCC,         "DCC"},
+ {ROAR_PROTO_ECHO,        "Echo"},
+ {ROAR_PROTO_DISCARD,     "Discard"},
+ {ROAR_PROTO_WHOIS,       "Whois"},
+ {ROAR_PROTO_FINGER,      "Finger"},
+ {ROAR_PROTO_QUOTE,       "quote"},
+ {ROAR_PROTO_DAYTIME,     "daytime"},
+ {-1, NULL}
+};
 
+int    roar_str2proto (const char * proto) {
+ size_t i;
+
+ for (i = 0; _proto[i].id != -1; i++)
+  if ( !strcasecmp(proto, _proto[i].name) )
+   return _proto[i].id;
+
+ roar_err_set(ROAR_ERROR_NOENT);
  return -1;
 }
 
-char * roar_proto2str (int    proto) {
- switch (proto) {
-  case ROAR_PROTO_ROARAUDIO: return "RoarAudio"; break;
-  case ROAR_PROTO_ESOUND:    return "EsounD";    break;
-  case ROAR_PROTO_AUTO:      return "(auto)";    break;
-  case ROAR_PROTO_HTTP:      return "http";      break;
-  case ROAR_PROTO_GOPHER:    return "gopher";    break;
-  case ROAR_PROTO_ICY:       return "ICY";       break;
-  case ROAR_PROTO_SIMPLE:    return "Simple";    break;
-  case ROAR_PROTO_RSOUND:    return "RSound";    break;
-  case ROAR_PROTO_RPLAY:     return "RPlay";     break;
-  default:
-    return "(unknown)";
- }
+char * roar_proto2str (const int    proto) {
+ size_t i;
+
+ roar_err_set(ROAR_ERROR_NONE);
+
+ for (i = 0; _proto[i].id != -1; i++)
+  if ( proto == _proto[i].id )
+   return (char*)_proto[i].name;
+
+ roar_err_set(ROAR_ERROR_NOENT);
+ return "(unknown)";
 }
 
-int    roar_str2byteorder (char * byteorder) {
+int    roar_str2byteorder (const char * byteorder) {
  if (        !strcasecmp(byteorder, "le")            || !strcasecmp(byteorder, "little") ||
              !strcasecmp(byteorder, "little endian") || !strcasecmp(byteorder, "1234")   ) {
   return ROAR_BYTEORDER_LE;
@@ -542,10 +618,11 @@ int    roar_str2byteorder (char * byteorder) {
   return ROAR_BYTEORDER_NETWORK;
  }
 
+ roar_err_set(ROAR_ERROR_NOENT);
  return -1;
 }
 
-char * roar_byteorder2str (int    byteorder) {
+char * roar_byteorder2str (const int    byteorder) {
  switch (byteorder) {
   case ROAR_BYTEORDER_LE:      return "little endian"; break;
   case ROAR_BYTEORDER_BE:      return "big endian";    break;
@@ -556,4 +633,261 @@ char * roar_byteorder2str (int    byteorder) {
  }
 }
 
+// grep '^#define ROAR_OT_' roaraudio/proto.h | cut -d' ' -f2 | sed 's/^\(ROAR_OT_\)\(.*\)$/ {\1\2, "\2"},/'
+static struct {
+ const int ot;
+ const char * name;
+} _libroar_ot[] = {
+ {ROAR_OT_CLIENT,   "client"},
+ {ROAR_OT_STREAM,   "stream"},
+ {ROAR_OT_SOURCE,   "source"},
+ {ROAR_OT_SAMPLE,   "sample"},
+ {ROAR_OT_OUTPUT,   "output"},
+ {ROAR_OT_MIXER,    "mixer"},
+ {ROAR_OT_BRIDGE,   "bridge"},
+ {ROAR_OT_LISTEN,   "listen"},
+ {ROAR_OT_ACTION,   "action"},
+ {ROAR_OT_MSGQUEUE, "msgqueue"},
+ {ROAR_OT_MSGBUS,   "msgbus"},
+ {-1, NULL}
+};
+
+int    roar_str2ot        (const char * ot) {
+ int i;
+
+ for (i = 0; _libroar_ot[i].ot != -1; i++)
+  if ( !strcasecmp(ot, _libroar_ot[i].name) )
+   return _libroar_ot[i].ot;
+
+ roar_err_set(ROAR_ERROR_NOENT);
+ return -1;
+}
+
+const char * roar_ot2str  (const int    ot) {
+ int i;
+
+ for (i = 0; _libroar_ot[i].ot != -1; i++)
+  if ( _libroar_ot[i].ot == ot )
+   return _libroar_ot[i].name;
+
+ roar_err_set(ROAR_ERROR_NOENT);
+ return NULL;
+}
+
+int roar_conv_volume (struct roar_mixer_settings * dst, struct roar_mixer_settings * src, int dstchans, int srcchans) {
+ struct roar_mixer_settings lsrc;
+ int i;
+ uint_least32_t s;
+
+ if ( dst == NULL || src == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ if ( dstchans < 0 || srcchans < 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return -1;
+ }
+
+ if ( dstchans == srcchans ) {
+  if ( dst == src )
+   return 0;
+
+  memcpy(dst, src, sizeof(struct roar_mixer_settings));
+  return 0;
+ }
+
+ if ( dst == src ) {
+  memcpy(&lsrc, src, sizeof(lsrc));
+  src = &lsrc;
+ }
+
+ // sepcal handling for N->1:
+ if ( dstchans == 1 ) {
+  s = 0;
+
+  for (i = 0; i < srcchans; i++)
+   s += src->mixer[i];
+
+  dst->mixer[0] = s / srcchans;
+ } else {
+  switch (srcchans) {
+   case  1:
+     for (i = 0; i < dstchans; i++)
+      dst->mixer[i] = src->mixer[0];
+    break;
+   case 2:
+     switch (dstchans) {
+      case 3:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = (src->mixer[0] + src->mixer[1]) / 2;
+       break;
+      case 4:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[0];
+        dst->mixer[3] = src->mixer[1];
+       break;
+      case 5:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = (src->mixer[0] + src->mixer[1]) / 2;
+        dst->mixer[3] = src->mixer[0];
+        dst->mixer[4] = src->mixer[1];
+       break;
+      case 6:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = (src->mixer[0] + src->mixer[1]) / 2;
+        dst->mixer[3] = dst->mixer[2];
+        dst->mixer[4] = src->mixer[0];
+        dst->mixer[5] = src->mixer[1];
+       break;
+      default:
+        roar_err_set(ROAR_ERROR_NOTSUP);
+        return -1;
+       break;
+     }
+    break;
+   case 3:
+     switch (dstchans) {
+      case 2:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+       break;
+      case 4:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[0];
+        dst->mixer[3] = src->mixer[1];
+       break;
+      case 5:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[2];
+        dst->mixer[3] = src->mixer[0];
+        dst->mixer[4] = src->mixer[1];
+       break;
+      case 6:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[2];
+        dst->mixer[3] = src->mixer[2];
+        dst->mixer[4] = src->mixer[0];
+        dst->mixer[5] = src->mixer[1];
+       break;
+      default:
+        roar_err_set(ROAR_ERROR_NOTSUP);
+        return -1;
+       break;
+     }
+    break;
+   case 4:
+     switch (dstchans) {
+      case 2:
+        dst->mixer[0] = (src->mixer[0] + src->mixer[2]) / 2;
+        dst->mixer[1] = (src->mixer[1] + src->mixer[3]) / 2;
+       break;
+      case 3:
+        dst->mixer[0] = (src->mixer[0] + src->mixer[2]) / 2;
+        dst->mixer[1] = (src->mixer[1] + src->mixer[3]) / 2;
+        dst->mixer[2] = (dst->mixer[0] + dst->mixer[1]) / 2;
+       break;
+      case 5:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = (src->mixer[0] + src->mixer[2] + src->mixer[1] + src->mixer[3]) / 4;
+        dst->mixer[3] = src->mixer[2];
+        dst->mixer[4] = src->mixer[3];
+       break;
+      case 6:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = (src->mixer[0] + src->mixer[2] + src->mixer[1] + src->mixer[3]) / 4;
+        dst->mixer[3] = dst->mixer[2];
+        dst->mixer[4] = src->mixer[2];
+        dst->mixer[5] = src->mixer[3];
+       break;
+      default:
+        roar_err_set(ROAR_ERROR_NOTSUP);
+        return -1;
+       break;
+     }
+    break;
+   case 5:
+     switch (dstchans) {
+      case 2:
+        dst->mixer[0] = (src->mixer[0] + src->mixer[3]) / 2;
+        dst->mixer[1] = (src->mixer[1] + src->mixer[4]) / 2;
+       break;
+      case 3:
+        dst->mixer[0] = (src->mixer[0] + src->mixer[3]) / 2;
+        dst->mixer[1] = (src->mixer[1] + src->mixer[4]) / 2;
+        dst->mixer[2] = src->mixer[2];
+       break;
+      case 4:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[3];
+        dst->mixer[3] = src->mixer[4];
+       break;
+      case 6:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[2];
+        dst->mixer[3] = src->mixer[2];
+        dst->mixer[4] = src->mixer[3];
+        dst->mixer[5] = src->mixer[4];
+       break;
+      default:
+        roar_err_set(ROAR_ERROR_NOTSUP);
+        return -1;
+       break;
+     }
+    break;
+   case 6:
+     switch (dstchans) {
+      case 2:
+        dst->mixer[0] = (src->mixer[0] + src->mixer[4]) / 2;
+        dst->mixer[1] = (src->mixer[1] + src->mixer[5]) / 2;
+       break;
+      case 3:
+        dst->mixer[0] = (src->mixer[0] + src->mixer[4]) / 2;
+        dst->mixer[1] = (src->mixer[1] + src->mixer[5]) / 2;
+        dst->mixer[2] = src->mixer[2];
+       break;
+      case 4:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[4];
+        dst->mixer[3] = src->mixer[5];
+       break;
+      case 5:
+        dst->mixer[0] = src->mixer[0];
+        dst->mixer[1] = src->mixer[1];
+        dst->mixer[2] = src->mixer[2];
+        dst->mixer[3] = src->mixer[4];
+        dst->mixer[4] = src->mixer[5];
+       break;
+      default:
+        roar_err_set(ROAR_ERROR_NOTSUP);
+        return -1;
+       break;
+     }
+    break;
+   default:
+     roar_err_set(ROAR_ERROR_NOTSUP);
+     return -1;
+    break;
+  }
+ }
+
+ dst->scale   = src->scale;
+ dst->rpg_mul = src->rpg_mul;
+ dst->rpg_div = src->rpg_div;
+
+ return 0;
+}
+
 //ll
