@@ -1,7 +1,7 @@
 //meta.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of libroar a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -41,9 +41,9 @@ grep ^'#define ROAR_META_TYPE_' meta.h | cut -d' ' -f2 | while read line; do pri
 
 */
 
-struct {
+static struct {
  int    id;
- char * name;
+ const char * name;
 } _libroar_meta_typelist[] = {
  {ROAR_META_TYPE_NONE,               "NONE"           },
  {ROAR_META_TYPE_TITLE,              "TITLE"          },
@@ -74,6 +74,7 @@ struct {
  {ROAR_META_TYPE_WOAF,               "WOAF"           },
  {ROAR_META_TYPE_ENCODER,            "ENCODER"        },
  {ROAR_META_TYPE_ENCODEDBY,          "ENCODEDBY"      },
+ {ROAR_META_TYPE_ENCODEDBY,          "ENCODED-BY"     }, // alias
  {ROAR_META_TYPE_YEAR,               "YEAR"           },
  {ROAR_META_TYPE_DISCID,             "DISCID"         },
  {ROAR_META_TYPE_RPG_TRACK_PEAK,     "REPLAYGAIN_TRACK_PEAK" },
@@ -86,14 +87,29 @@ struct {
  {ROAR_META_TYPE_OFFSET,             "OFFSET"         },
  {ROAR_META_TYPE_PERFORMER,          "PERFORMER"      },
  {ROAR_META_TYPE_COPYRIGHT,          "COPYRIGHT"      },
+ {ROAR_META_TYPE_LIKENESS,           "LIKENESS"       },
+ {ROAR_META_TYPE_COMPOSER,           "COMPOSER"       },
+ {ROAR_META_TYPE_RIGHTS,             "RIGHTS"         },
+ {ROAR_META_TYPE_ISRC,               "ISRC"           },
+ {ROAR_META_TYPE_LANGUAGE,           "LANGUAGE"       },
+ {ROAR_META_TYPE_GTIN,               "GTIN"           },
+ {ROAR_META_TYPE_GTIN,               "EAN/UPN"        }, // alias
+ {ROAR_META_TYPE_GTIN,               "UPN"            }, // alias
+ {ROAR_META_TYPE_ISBN,               "ISBN"           },
+ {ROAR_META_TYPE_EAN,                "EAN"            },
+ {ROAR_META_TYPE_PUBLISHER,          "PUBLISHER"      },
+ {ROAR_META_TYPE_DISCNUMBER,         "DISCNUMBER"     },
+ {ROAR_META_TYPE_SOURCEMEDIA,        "SOURCEMEDIA"    },
+ {ROAR_META_TYPE_LABEL,              "LABEL"          },
+ {ROAR_META_TYPE_LABELNO,            "LABELNO"        },
  
  {-1, "EOL"}
 };
 
 
-struct {
+static struct {
  int    id;
- char * name;
+ const char * name;
 } _libroar_meta_genrelist[] = {
 {ROAR_META_GENRE_RDS_EU_NONE,             "rds_eu_none"},
 {ROAR_META_GENRE_RDS_EU_NONE,             "none"},
@@ -395,7 +411,8 @@ struct {
  {-1, "EOL"}
 };
 
-char * roar_meta_strtype(int type) {
+// TODO: Why do we return a non-const buffer here?
+char * roar_meta_strtype(const int type) {
  int i;
  static char name[ROAR_META_MAX_NAMELEN];
 
@@ -408,7 +425,7 @@ char * roar_meta_strtype(int type) {
  return NULL;
 }
 
-int    roar_meta_inttype(char * type) {
+int    roar_meta_inttype(const char * type) {
  int i;
 
  for (i = 0; _libroar_meta_typelist[i].id != -1; i++)
@@ -544,7 +561,7 @@ int roar_meta_free (struct roar_meta * meta) {
 
 
 // genere:
-char * roar_meta_strgenre(int genre) {
+const char * roar_meta_strgenre(const int genre) {
  int i;
 
  for (i = 0; _libroar_meta_genrelist[i].id != -1; i++)
@@ -555,7 +572,7 @@ char * roar_meta_strgenre(int genre) {
  return NULL;
 }
 
-int    roar_meta_intgenre(char * genre) {
+int    roar_meta_intgenre(const char * genre) {
  int i;
 
  for (i = 0; _libroar_meta_genrelist[i].id != -1; i++)
@@ -566,7 +583,7 @@ int    roar_meta_intgenre(char * genre) {
  return -1;
 }
 
-int    roar_meta_parse_audioinfo(struct roar_audio_info * info, char * str) {
+int    roar_meta_parse_audioinfo(struct roar_audio_info * info, const char * str) {
  char * lc;
  char * cur, * next;
  char * k,   * v;
@@ -578,7 +595,7 @@ int    roar_meta_parse_audioinfo(struct roar_audio_info * info, char * str) {
 
  info->codec = -1;
 
- if ( (lc = strdup(str)) == NULL )
+ if ( (lc = roar_mm_strdup(str)) == NULL )
   return -1;
 
  cur = lc;
@@ -614,7 +631,7 @@ int    roar_meta_parse_audioinfo(struct roar_audio_info * info, char * str) {
   cur = next;
  }
 
- free(lc);
+ roar_mm_free(lc);
 
  return 0;
 }
