@@ -1,7 +1,7 @@
 //socket.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of libroar a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -33,6 +33,7 @@
  *  them with any software that uses libesd, libartsc or libpulse*.
  */
 
+#define _LIBROAR_NOATTR_TO_STATIC /* ignore warnings for TO_STATIC functions */
 #include "libroar.h"
 
 #define MODE_LISTEN  ROAR_SOCKET_MODE_LISTEN
@@ -55,9 +56,6 @@ void roar_socket_win32_init (void) {
 int roar_socket_new_tcp (void) {
 #ifdef ROAR_HAVE_IPV4
  int fh;
-#ifndef ROAR_TARGET_WIN32
- int opt = IPTOS_LOWDELAY;
-#endif
 #if defined(TCP_NODELAY) && !defined(ROAR_TARGET_WIN32)
  int t   = 1;
 #endif
@@ -66,9 +64,7 @@ int roar_socket_new_tcp (void) {
 
  fh = socket(AF_INET, SOCK_STREAM, 0);
 
-#ifndef ROAR_TARGET_WIN32
- setsockopt(fh, IPPROTO_IP, IP_TOS, &opt, sizeof(int));
-#endif
+ roar_socket_set_tos(fh);
 #if defined(TCP_NODELAY) && !defined(ROAR_TARGET_WIN32)
  setsockopt(fh, IPPROTO_TCP, TCP_NODELAY, &t, sizeof(int));
 #endif
@@ -82,17 +78,12 @@ int roar_socket_new_tcp (void) {
 int roar_socket_new_udp (void) {
 #ifdef ROAR_HAVE_IPV4
  int fh;
-#ifndef ROAR_TARGET_WIN32
- int opt = IPTOS_LOWDELAY;
-#endif
 
  roar_socket_win32_init();
 
  fh = socket(AF_INET, SOCK_DGRAM, 0);
 
-#ifndef ROAR_TARGET_WIN32
- setsockopt(fh, IPPROTO_IP, IP_TOS, &opt, sizeof(int));
-#endif
+ roar_socket_set_tos(fh);
 
  return fh;
 #else
@@ -103,13 +94,12 @@ int roar_socket_new_udp (void) {
 int roar_socket_new_tcp6 (void) {
 #ifdef ROAR_HAVE_IPV6
  int fh;
- int opt = IPTOS_LOWDELAY;
 
  roar_socket_win32_init();
 
  fh = socket(AF_INET6, SOCK_STREAM, 0);
 
- setsockopt(fh, IPPROTO_IP, IP_TOS, &opt, sizeof(int));
+ roar_socket_set_tos(fh);
 
  return fh;
 #else
@@ -120,13 +110,12 @@ int roar_socket_new_tcp6 (void) {
 int roar_socket_new_udp6 (void) {
 #ifdef ROAR_HAVE_IPV6
  int fh;
- int opt = IPTOS_LOWDELAY;
 
  roar_socket_win32_init();
 
  fh = socket(AF_INET6, SOCK_DGRAM, 0);
 
- setsockopt(fh, IPPROTO_IP, IP_TOS, &opt, sizeof(int));
+ roar_socket_set_tos(fh);
 
  return fh;
 #else
@@ -171,7 +160,23 @@ int roar_socket_recvbuf(int fh, int len) {
 #ifdef ROAR_HAVE_BSDSOCKETS
  if ( len < 256 ) len = 256;
 
- return setsockopt(fh, SOL_SOCKET, SO_RCVBUF, &len, sizeof(len));
+ return setsockopt(fh, SOL_SOCKET, SO_RCVBUF, (void *)&len, sizeof(len));
+#else
+ return -1;
+#endif
+}
+
+int roar_socket_set_tos(int fh) {
+#if defined(ROAR_HAVE_BSDSOCKETS) && !defined(ROAR_TARGET_WIN32)
+ int opt = IPTOS_LOWDELAY;
+ int ret;
+
+ roar_err_clear_errno();
+ ret = setsockopt(fh, IPPROTO_IP, IP_TOS, &opt, sizeof(int));
+ if ( ret < 0 )
+  roar_err_from_errno();
+
+ return ret;
 #else
  return -1;
 #endif
@@ -220,10 +225,11 @@ int roar_socket_new_ipx    (void) {
 
 
 int roar_socket_nonblock(int fh, int state) {
-#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER)
+#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER) && defined(ROAR_HAVE_FCNTL)
  int flags;
 
  if ( (flags = fcntl(fh, F_GETFL, 0)) == -1 ) {
+  roar_err_from_errno();
   ROAR_ERR("roar_socket_nonblock(fh=%i, state=%i): Can not read flags: %s", fh, state, strerror(errno));
   ROAR_DBG("roar_socket_nonblock(fh=%i, state=%i) = -1", fh, state);
   return -1;
@@ -238,6 +244,7 @@ int roar_socket_nonblock(int fh, int state) {
   flags -= O_NONBLOCK;
 
  if ( fcntl(fh, F_SETFL, flags) == -1 ) {
+  roar_err_from_errno();
   ROAR_ERR("roar_socket_nonblock(fh=%i, state=%i): Can not set flags: %s", fh, state, strerror(errno));
   ROAR_DBG("roar_socket_nonblock(fh=%i, state=%i) = -1", fh, state);
   return -1;
@@ -252,7 +259,7 @@ int roar_socket_nonblock(int fh, int state) {
 }
 
 int roar_socket_dup_udp_local_end (int fh) {
-#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER)
+#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER) && defined(ROAR_HAVE_FCNTL)
  int                  n              = -1;
  int                  flags          = -1;
  struct sockaddr_in   socket_addr;
@@ -263,10 +270,12 @@ int roar_socket_dup_udp_local_end (int fh) {
  }
 
  if ( getsockname(fh, (struct sockaddr *)&socket_addr, &len) == -1 ) {
+  roar_err_from_errno();
   return -1;
  }
 
  if ( socket_addr.sin_family != AF_INET ) {
+  roar_err_set(ROAR_ERROR_TYPEMM);
   return -1;
  }
 
@@ -277,6 +286,7 @@ int roar_socket_dup_udp_local_end (int fh) {
 
 //  if ( mode_func(fh, (struct sockaddr *)&socket_addr, sizeof(struct sockaddr_in)) == -1 ) {
  if ( bind(n, (struct sockaddr *)&socket_addr, len) == -1 ) {
+  roar_err_from_errno();
   close(n);
   return -1;
  }
@@ -299,7 +309,7 @@ int roar_socket_dup_udp_local_end (int fh) {
 
 #define _SCMR_CONTROLLEN (sizeof(struct cmsghdr) + sizeof(int))
 int roar_socket_send_fh (int sock, int fh, char * mes, size_t len) {
-#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER) && !defined(ROAR_OS_SUNOS)
+#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER) && !defined(ROAR_OS_SUNOS) && !defined(ROAR_TARGET_OPENVMS)
  struct iovec     iov[1];
  struct msghdr    msg;
  char             cmptr_buf[_SCMR_CONTROLLEN];
@@ -340,7 +350,7 @@ int roar_socket_send_fh (int sock, int fh, char * mes, size_t len) {
 }
 
 int roar_socket_recv_fh (int sock,         char * mes, size_t * len) {
-#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER) && !defined(ROAR_OS_SUNOS)
+#if !defined(ROAR_TARGET_WIN32) && !defined(ROAR_TARGET_MICROCONTROLLER) && !defined(ROAR_OS_SUNOS) && !defined(ROAR_TARGET_OPENVMS)
  struct iovec     iov[1];
  struct msghdr    msg;
  char             cmptr_buf[_SCMR_CONTROLLEN];
@@ -348,8 +358,12 @@ int roar_socket_recv_fh (int sock,         char * mes, size_t * len) {
  char             localmes[1];
  size_t           locallen[1] = {1};
 
- if ( sock < 0 )
+ ROAR_DBG("roar_socket_recv_fh(sock=%i, mes=%p, len=%p) = ?", sock, mes, len);
+
+ if ( sock < 0 ) {
+  ROAR_DBG("roar_socket_recv_fh(sock=%i, mes=%p, len=%p) = -1 // invalid socket", sock, mes, len);
   return -1;
+ }
 
  if ( len == NULL ) {
   len = locallen;
@@ -366,12 +380,18 @@ int roar_socket_recv_fh (int sock,         char * mes, size_t * len) {
  msg.msg_control    = (caddr_t) cmptr;
  msg.msg_controllen = _SCMR_CONTROLLEN;
 
- if ( (*len = recvmsg(sock, &msg, 0)) == -1 )
+ if ( (*len = recvmsg(sock, &msg, 0)) == -1 ) {
+  ROAR_DBG("roar_socket_recv_fh(sock=%i, mes=%p, len=%p) = -1 // can not read from socket", sock, mes, len);
   return -1;
+ }
 
- if ( msg.msg_controllen != _SCMR_CONTROLLEN )
+ if ( msg.msg_controllen  < _SCMR_CONTROLLEN ||
+      cmptr->cmsg_len    != _SCMR_CONTROLLEN  ) {
+  ROAR_DBG("roar_socket_recv_fh(sock=%i, mes=%p, len=%p) = -1 // control len is wrong", sock, mes, len);
   return -1;
+ }
 
+ ROAR_DBG("roar_socket_recv_fh(sock=%i, mes=%p, len=%p) = %i", sock, mes, len, *(int *)CMSG_DATA(cmptr));
  return *(int *)CMSG_DATA(cmptr);
 #else
  ROAR_ERR("roar_socket_recv_fh(*): There is no UNIX Domain Socket support in win32, download a real OS.");
@@ -386,6 +406,8 @@ int roar_socket_listen  (int type, char * host, int port) {
 int roar_socket_connect (char * host, int port) {
  char * proxy_type = getenv("ROAR_PROXY");
 
+ ROAR_DBG("roar_socket_connect(host='%s', port=%i) = ?", host, port);
+
  if ( proxy_type == NULL || strcmp(proxy_type, "") == 0 ) {
   return roar_socket_open(MODE_CONNECT, ROAR_SOCKET_TYPE_UNKNOWN, host, port);
  } else {
@@ -447,7 +469,7 @@ int roar_socket_listen_decnet (char * object, int num) {
 #endif
 }
 
-char * roar_socket_get_local_nodename(void) {
+const char * roar_socket_get_local_nodename(void) {
 #ifdef ROAR_HAVE_LIBDNET
  static char node[16] = {0};
  struct dn_naddr      *binaddr;
@@ -475,6 +497,8 @@ int roar_socket_open (int mode, int type, char * host, int port) {
  int fh;
 #ifdef ROAR_HAVE_IPX
 #define _NEED_OBJ
+#endif
+#if defined(ROAR_HAVE_IPX) || defined(ROAR_HAVE_GETADDRINFO)
  int ret;
 #endif
 #ifdef ROAR_HAVE_IPX
@@ -499,6 +523,7 @@ int roar_socket_open (int mode, int type, char * host, int port) {
   struct sockaddr_ipx ipx;
 #endif
  } socket_addr;
+ socklen_t addrlen;
 #endif
 #if defined(ROAR_HAVE_IPV4) || defined(ROAR_HAVE_IPV6)
  struct hostent     * he;
@@ -516,11 +541,25 @@ int roar_socket_open (int mode, int type, char * host, int port) {
  char obj[80];
  char * del;
 #endif
+ int af_guessed = 0;
+#ifdef ROAR_HAVE_GETADDRINFO
+ struct addrinfo hints, *res = NULL;
+ char port_as_string[32];
+#endif
+#if defined(ROAR_HAVE_IPV4) || defined(ROAR_HAVE_IPV6)
+ int is_numerical = 1;
+ const char * numptr;
+#endif
+
+ ROAR_DBG("roar_socket_open(mode=%i, type=%i, host='%s', port=%i) = ?", mode, type, host, port);
+
+ roar_err_set(ROAR_ERROR_UNKNOWN);
 
  if ( mode == MODE_LISTEN )
   mode_func = bind;
 
  if ( type == ROAR_SOCKET_TYPE_UNKNOWN ) {
+  af_guessed = 1;
   type = ROAR_SOCKET_TYPE_INET;
   if ( *host == '/' ) {
    type = ROAR_SOCKET_TYPE_UNIX;
@@ -539,6 +578,8 @@ int roar_socket_open (int mode, int type, char * host, int port) {
  }
 
 
+ ROAR_DBG("roar_socket_open(mode=%i, type=%i, host='%s', port=%i) = ?", mode, type, host, port);
+
  ROAR_DBG("roar_socket_open(*): type=%s, host='%s', port=%i",
              type == ROAR_SOCKET_TYPE_UNIX ? "UNIX" : "???", host, port);
 
@@ -550,6 +591,7 @@ int roar_socket_open (int mode, int type, char * host, int port) {
 
    if ( del == NULL ) {
     ROAR_WARN("roar_socket_open(*): invalid hostname for DECnet: %s", host);
+    roar_err_set(ROAR_ERROR_INVAL);
     return -1;
    }
 
@@ -566,18 +608,22 @@ int roar_socket_open (int mode, int type, char * host, int port) {
     strncat(obj, del+2, 79);
    }
 
+  ROAR_DBG("roar_socket_open(*): obj='%s', port=%i", obj, port);
+
   if ( mode == MODE_LISTEN ) {
    fh = roar_socket_listen_decnet(obj, port);
    *del = ':';
    return fh;
-//   return -1; // listen sockets on DECnet are not supportet at the moment
   } else {
    // There is nothing wrong in this case to use dnet_conn() so we do.
+   ROAR_DBG("roar_socket_open(*): CALL dnet_conn('%s', '%s', SOCK_STREAM, 0 ,0 ,0 , 0)", host, obj);
    fh = dnet_conn(host, obj, SOCK_STREAM, 0 ,0 ,0 , 0);
+   ROAR_DBG("roar_socket_open(*): RET %i", fh);
    *del = ':';
    return fh;
   }
 #else
+  roar_err_set(ROAR_ERROR_AFNOTSUP);
   return -1; // no decnet support
 #endif
  }
@@ -597,19 +643,88 @@ int roar_socket_open (int mode, int type, char * host, int port) {
 
   ROAR_DBG("roar_socket_open(*): type=INET|INET6, host='%s', port=%i", host, port);
 
+  if ( !!strcmp(host, "0.0.0.0") ) {
+   for (numptr = host; is_numerical && *numptr != 0; numptr++)
+    if ( ! ((*numptr >= '0' && *numptr <= '9') || *numptr == '.')  )
+     is_numerical = 0;
+
+   if ( is_numerical && roar_libroar_iswarn(NULL) ) {
+    ROAR_WARN("roar_socket_open(*): Hostname \"%s\" is numerical. Do not use IP addresses directly. Use Hostnames.", host);
+   }
+  }
+
+#ifdef ROAR_HAVE_GETADDRINFO
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_socktype = SOCK_STREAM;
+  if ( af_guessed ) {
+   hints.ai_family   = AF_UNSPEC;
+  } else {
+   hints.ai_family   = type == ROAR_SOCKET_TYPE_INET ? AF_INET : AF_INET6;
+  }
+
+  snprintf(port_as_string, sizeof(port_as_string), "%i", port);
+
+  ret = getaddrinfo(host, port_as_string, &hints, &res);
+  switch (ret) {
+   case 0: /* no error */; break;
+#ifdef EAI_ADDRFAMILY
+   case EAI_ADDRFAMILY: roar_err_set(ROAR_ERROR_NOENT);    break;
+#endif
+   case EAI_AGAIN:      roar_err_set(ROAR_ERROR_AGAIN);    break;
+   case EAI_BADFLAGS:   roar_err_set(ROAR_ERROR_INVAL);    break;
+   case EAI_FAIL:       roar_err_set(ROAR_ERROR_RIO);      break;
+   case EAI_FAMILY:     roar_err_set(ROAR_ERROR_AFNOTSUP); break;
+   case EAI_MEMORY:     roar_err_set(ROAR_ERROR_NOMEM);    break;
+#ifdef EAI_NODATA
+   case EAI_NODATA:     roar_err_set(ROAR_ERROR_NODATA);   break;
+#endif
+   case EAI_NONAME:     roar_err_set(ROAR_ERROR_NOENT);    break;
+   case EAI_SERVICE:    roar_err_set(ROAR_ERROR_PROTONOSUP); break;
+   case EAI_SOCKTYPE:   roar_err_set(ROAR_ERROR_INVAL);    break;
+   case EAI_SYSTEM:     roar_err_from_errno();             break;
+   default:
+     roar_err_set(ROAR_ERROR_UNKNOWN);
+    break;
+  }
+  if ( ret != 0 )
+   return -1;
+
+  if ( af_guessed ) {
+   type = res->ai_family == AF_INET ? ROAR_SOCKET_TYPE_INET : ROAR_SOCKET_TYPE_INET6;
+  }
+
+  if ( type == ROAR_SOCKET_TYPE_INET ) {
+   fh = roar_socket_new_tcp();
+  } else {
+   fh = roar_socket_new_tcp6();
+  }
+
+  memcpy(&(socket_addr.sa), res->ai_addr, res->ai_addrlen);
+  addrlen = res->ai_addrlen;
+
+  if ( res != NULL )
+   freeaddrinfo(res);
+#else
   if ( (he = gethostbyname(host)) == NULL ) {
    ROAR_ERR("roar_socket_open(*): Can\'t resolve host name '%s'",
                      host);
+   roar_err_from_errno();
    return -1;
   }
 
-   memcpy((struct in_addr *)&socket_addr.in.sin_addr, he->h_addr, sizeof(struct in_addr));
+   ROAR_DBG("roar_socket_open(*): he=%p", he);
+
+   memcpy((struct in_addr *)&(socket_addr.in.sin_addr), he->h_addr, sizeof(struct in_addr));
 
    /* set the connect information */
    socket_addr.in.sin_family = AF_INET;
    socket_addr.in.sin_port   = ROAR_HOST2NET16(port);
 
   fh = roar_socket_new_tcp();
+  addrlen = sizeof(struct sockaddr_in);
+#endif
+
+  ROAR_DBG("roar_socket_open(*): fh=%i", fh);
 
   if ( fh == -1 ) {
    ROAR_ERR("roar_socket_open(*): Can\'t create TCP socket: %s", strerror(errno));
@@ -618,14 +733,22 @@ int roar_socket_open (int mode, int type, char * host, int port) {
 
   ROAR_DBG("roar_socket_open(*) = ?");
 
-   if ( mode_func(fh, (struct sockaddr *)&socket_addr.in, sizeof(struct sockaddr_in)) == -1 ) {
+  ROAR_DBG("roar_socket_open(*): fh=%i", fh);
+
+   if ( mode_func(fh, (struct sockaddr *)&socket_addr.sa, addrlen) == -1 ) {
     ROAR_DBG("roar_socket_open(*): Can not connect/bind: %s", strerror(errno));
+    roar_err_from_errno();
     close(fh);
     return -1;
    }
+
+  ROAR_DBG("roar_socket_open(*): fh=%i", fh);
+
   // hey! we have a socket...
   ROAR_DBG("roar_socket_open(*) = ? // we have a socket :)");
 #else
+  ROAR_DBG("roar_socket_open(*) = -1 // no IPv4 or IPv6 support");
+  roar_err_set(ROAR_ERROR_AFNOTSUP);
   return -1;
 #endif
  } else if ( type == ROAR_SOCKET_TYPE_UNIX ) {
@@ -643,11 +766,13 @@ int roar_socket_open (int mode, int type, char * host, int port) {
 
   if ( mode_func(fh, (struct sockaddr *)&socket_addr.un, sizeof(struct sockaddr_un)) == -1 ) {
    ROAR_DBG("roar_socket_open(*): Can not connect/bind: %s", strerror(errno));
+   roar_err_from_errno();
    close(fh);
    return -1;
   }
 #else
   ROAR_ERR("roar_socket_open(*): There is no UNIX Domain Socket support in win32, download a real OS.");
+  roar_err_set(ROAR_ERROR_AFNOTSUP);
   return -1;
 #endif
  } else if ( type == ROAR_SOCKET_TYPE_IPX ) {
@@ -672,8 +797,10 @@ int roar_socket_open (int mode, int type, char * host, int port) {
   fh = roar_socket_new_ipx();
 
   close(fh);
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
 #else
+  roar_err_set(ROAR_ERROR_AFNOTSUP);
   return -1;
 #endif
  } else if ( type == ROAR_SOCKET_TYPE_FORK ) {
@@ -681,16 +808,19 @@ int roar_socket_open (int mode, int type, char * host, int port) {
  } else if ( type == ROAR_SOCKET_TYPE_FILE ) {
   return roar_socket_open_file(mode, host, port);
  } else {
+  roar_err_set(ROAR_ERROR_AFNOTSUP);
   return -1;
  }
 
  if ( mode == MODE_LISTEN ) {
 #if defined(ROAR_HAVE_BSDSOCKETS) || defined(ROAR_TARGET_WIN32)
   if ( listen(fh, ROAR_SOCKET_QUEUE_LEN) == -1 ) {
+   roar_err_from_errno();
    close(fh);
    return -1;
   }
 #else
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
 #endif
  }
