@@ -1,7 +1,7 @@
 //stream.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of libroar a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -35,7 +35,11 @@
 
 #include "libroar.h"
 
-int roar_stream_connect (struct roar_connection * con, struct roar_stream * s, int dir) {
+int roar_stream_connect  (struct roar_connection * con, struct roar_stream * s, int dir) {
+ return roar_stream_connect2(con, s, dir, -1);
+}
+
+int roar_stream_connect2 (struct roar_connection * con, struct roar_stream * s, int dir, int mixer) {
  struct roar_libroar_config * config = roar_libroar_get_config();
  struct roar_stream  ms;
  struct roar_message m;
@@ -46,7 +50,7 @@ int roar_stream_connect (struct roar_connection * con, struct roar_stream * s, i
  memcpy(&ms, s, sizeof(ms));
 
  m.cmd     = ROAR_CMD_NEW_STREAM;
- m.stream  = -1;
+ m.stream  = mixer;
  m.pos     = 0;
 
  if ( config != NULL ) {
@@ -62,7 +66,7 @@ int roar_stream_connect (struct roar_connection * con, struct roar_stream * s, i
 
  roar_stream_s2m(&ms, &m);
 
- if ( roar_req(con, &m, NULL) != 0 )
+ if ( roar_req3(con, &m, NULL) != 0 )
   return -1;
 
  if ( m.cmd == ROAR_CMD_OK ) {
@@ -80,8 +84,10 @@ int roar_stream_connect (struct roar_connection * con, struct roar_stream * s, i
 int roar_stream_new (struct roar_stream * s, unsigned int rate,
                      unsigned int channels, unsigned int bits, unsigned int codec) {
 
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  s->fh         = -1;
  s->id         = -1;
@@ -110,8 +116,10 @@ int roar_stream_new (struct roar_stream * s, unsigned int rate,
 }
 
 int roar_stream_set_rel_id(struct roar_stream * s, int id) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  s->pos_rel_id = id;
 
@@ -119,15 +127,19 @@ int roar_stream_set_rel_id(struct roar_stream * s, int id) {
 }
 
 int roar_stream_get_rel_id(struct roar_stream * s) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return s->pos_rel_id;
 }
 
 int roar_stream_new_by_id(struct roar_stream * s, int id) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  if ( roar_stream_new_empty(s) == -1 )
   return -1;
@@ -136,15 +148,19 @@ int roar_stream_new_by_id(struct roar_stream * s, int id) {
 }
 
 int roar_stream_new_empty(struct roar_stream * s) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return roar_stream_new(s, 0, 0, 0, 0);
 }
 
 int roar_stream_set_id (struct roar_stream * s, int id) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  s->id = id;
 
@@ -152,15 +168,19 @@ int roar_stream_set_id (struct roar_stream * s, int id) {
 }
 
 int roar_stream_get_id (struct roar_stream * s) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return s->id;
 }
 
 int roar_stream_set_fh (struct roar_stream * s, int fh) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  s->fh = fh;
 
@@ -168,15 +188,19 @@ int roar_stream_set_fh (struct roar_stream * s, int fh) {
 }
 
 int roar_stream_get_fh (struct roar_stream * s) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return s->fh;
 }
 
 int roar_stream_set_dir (struct roar_stream * s, int dir) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  s->dir = dir;
 
@@ -184,8 +208,10 @@ int roar_stream_set_dir (struct roar_stream * s, int dir) {
 }
 
 int roar_stream_get_dir (struct roar_stream * s) {
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return s->dir;
 }
@@ -227,8 +253,10 @@ int roar_stream_connect_to_ask (struct roar_connection * con, struct roar_stream
  struct roar_message m;
  int len = 0;
 
- if ( host == NULL )
+ if ( host == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  ROAR_DBG("roar_stream_connect_to_ask(*): Ask the server to connect to: %s:%i", host, port);
 
@@ -244,8 +272,10 @@ int roar_stream_connect_to_ask (struct roar_connection * con, struct roar_stream
 
  len = strlen(host);
 
- if ( len > 76 )
+ if ( len > 76 ) {
+  roar_err_set(ROAR_ERROR_NAMETOOLONG);
   return -1;
+ }
 
  strncpy(&(m.data[4]), host, len);
 
@@ -261,6 +291,16 @@ int roar_stream_passfh  (struct roar_connection * con, struct roar_stream * s, i
  struct roar_message m;
  int confh;
 
+ if ( con == NULL || s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ if ( fh < 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return -1;
+ }
+
  memset(&m,  0, sizeof(m));
 
  m.cmd     = ROAR_CMD_PASSFH;
@@ -305,6 +345,16 @@ int roar_stream_attach_simple (struct roar_connection * con, struct roar_stream
  uint16_t * info = (uint16_t *) m.data;
  int i;
 
+ if ( con == NULL || s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ if ( client < 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return -1;
+ }
+
  memset(&m,  0, sizeof(m));
 
  m.cmd     = ROAR_CMD_ATTACH;
@@ -332,6 +382,11 @@ int roar_stream_attach_simple (struct roar_connection * con, struct roar_stream
 int roar_stream_add_data (struct roar_connection * con, struct roar_stream * s, char * data, size_t len) {
  struct roar_message m;
 
+ if ( con == NULL || s == NULL || data == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
  memset(&m,  0, sizeof(m));
 
  m.cmd     = ROAR_CMD_ADD_DATA;
@@ -356,12 +411,16 @@ int roar_stream_send_data (struct roar_connection * con, struct roar_stream * s,
 
  roar_debug_warn_obsolete("roar_stream_send_data", "roar_vio_write", NULL);
 
- if ( s == NULL )
+ if ( s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  if ( s->fh == -1 ) {
-  if ( con == NULL )
+  if ( con == NULL ) {
+   roar_err_set(ROAR_ERROR_NOTSUP);
    return -1;
+  }
 
   if ( roar_stream_add_data(con, s, data, len) == -1 )
    return -1;
@@ -370,9 +429,10 @@ int roar_stream_send_data (struct roar_connection * con, struct roar_stream * s,
  }
 
 #ifdef ROAR_HAVE_IO_POSIX
- return write(s->fh, data, len);
+ return ROAR_NETWORK_WRITE(s->fh, data, len);
 #endif
 
+ roar_err_set(ROAR_ERROR_NOTSUP);
  return -1;
 }
 
@@ -381,6 +441,11 @@ int roar_stream_get_info (struct roar_connection * con, struct roar_stream * s,
  uint16_t * data = (uint16_t *) m.data;
  int i;
 
+ if ( con == NULL || s == NULL || info == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
  memset(&m,  0, sizeof(m));
 
  m.cmd     = ROAR_CMD_GET_STREAM_PARA;
@@ -405,11 +470,15 @@ int roar_stream_get_info (struct roar_connection * con, struct roar_stream * s,
   data[i] = ROAR_NET2HOST16(data[i]);
  }
 
- if ( m.datalen < 7*2 )
+ if ( m.datalen < 7*2 ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
- if ( data[0] != 0 || data[1] != 1 )
+ if ( data[0] != 0 || data[1] != 1 ) {
+  roar_err_set(ROAR_ERROR_NSVERSION);
   return -1;
+ }
 
  memset(info, 0, sizeof(struct roar_stream_info));
  info->mixer = -1;
@@ -454,8 +523,10 @@ int roar_stream_get_name (struct roar_connection * con, struct roar_stream * s,
  struct roar_message m;
  uint16_t * data = (uint16_t *) m.data;
 
- if ( con == NULL || s == NULL || name == NULL || len == 0 )
+ if ( con == NULL || s == NULL || name == NULL || len == 0 ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  name[0] = 0; // just in case...
 
@@ -484,16 +555,25 @@ int roar_stream_get_name (struct roar_connection * con, struct roar_stream * s,
 
  ROAR_DBG("roar_stream_get_name(*) = ?");
 
- if ( m.datalen < 4 )
+ if ( m.datalen < 4 ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
  data[0] = ROAR_NET2HOST16(data[0]);
  data[1] = ROAR_NET2HOST16(data[1]);
 
  ROAR_DBG("roar_stream_get_name(*) = ?");
 
- if ( data[0] != 0 || data[1] != ROAR_STREAM_PARA_NAME )
+ if ( data[0] != 0 ) {
+  roar_err_set(ROAR_ERROR_NSVERSION);
   return -1;
+ }
+
+ if ( data[1] != ROAR_STREAM_PARA_NAME ) {
+  roar_err_set(ROAR_ERROR_TYPEMM);
+  return -1;
+ }
 
  m.datalen -= 4;
 
@@ -516,11 +596,15 @@ int roar_stream_get_chanmap (struct roar_connection * con, struct roar_stream *
 
  ROAR_DBG("roar_stream_get_chanmap(con=%p, s=%p, map=%p, len=%p) = ?", con, s, map, len);
 
- if ( con == NULL || s == NULL || map == NULL || len == NULL )
+ if ( con == NULL || s == NULL || map == NULL || len == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( *len == 0 )
+ if ( *len == 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
   return -1;
+ }
 
  memset(&m, 0, sizeof(m));
 
@@ -544,23 +628,34 @@ int roar_stream_get_chanmap (struct roar_connection * con, struct roar_stream *
 
  ROAR_DBG("roar_stream_get_chanmap(con=%p, s=%p{.id=%i}, map=%p, len=%p) = ?", con, s, s->id, map, len);
 
- if ( m.datalen < 4 )
+ if ( m.datalen < 4 ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
  data[0] = ROAR_NET2HOST16(data[0]);
  data[1] = ROAR_NET2HOST16(data[1]);
 
  ROAR_DBG("roar_stream_get_chanmap(con=%p, s=%p{.id=%i}, map=%p, len=%p) = ?", con, s, s->id, map, len);
 
- if ( data[0] != 0 || data[1] != ROAR_STREAM_PARA_CHANMAP )
+ if ( data[0] != 0 ) {
+  roar_err_set(ROAR_ERROR_NSVERSION);
   return -1;
+ }
+
+ if ( data[1] != ROAR_STREAM_PARA_CHANMAP ) {
+  roar_err_set(ROAR_ERROR_TYPEMM);
+  return -1;
+ }
 
  ROAR_DBG("roar_stream_get_chanmap(con=%p, s=%p{.id=%i}, map=%p, len=%p) = ?", con, s, s->id, map, len);
 
  m.datalen -= 4;
 
- if ( m.datalen > *len )
+ if ( m.datalen > *len ) {
+  roar_err_set(ROAR_ERROR_NOMEM);
   return -1;
+ }
 
  ROAR_DBG("roar_stream_get_chanmap(con=%p, s=%p{.id=%i}, map=%p, len=%p) = ?", con, s, s->id, map, len);
 
@@ -576,11 +671,15 @@ int roar_stream_set_chanmap (struct roar_connection * con, struct roar_stream *
  struct roar_message m;
  uint16_t * data = (uint16_t *) m.data;
 
- if ( con == NULL || s == NULL || map == NULL )
+ if ( con == NULL || s == NULL || map == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( len == 0 )
+ if ( len == 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
   return 0;
+ }
 
  memset(&m, 0, sizeof(m));
 
@@ -610,21 +709,39 @@ int roar_stream_set_chanmap (struct roar_connection * con, struct roar_stream *
 
 
 int roar_stream_set_flags (struct roar_connection * con, struct roar_stream * s, int flags, int reset) {
+ return roar_stream_set_flags2(con, s, flags, reset == ROAR_RESET_FLAG ? ROAR_RESET_FLAG : ROAR_SET_FLAG);
+}
+
+int roar_stream_set_flags2 (struct roar_connection * con, struct roar_stream * s, uint32_t flags, int action) {
  struct roar_message m;
  uint16_t * data = (uint16_t *) m.data;
  int i;
 
+ if ( con == NULL || s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
  memset(&m,  0, sizeof(m));
 
  m.cmd     = ROAR_CMD_SET_STREAM_PARA;
  m.stream  = s->id;
- m.datalen = 8;
  m.pos     = 0;
 
+ if ( flags & 0xFFFF0000 ) {
+  m.datalen = 2*5;
+ } else {
+  m.datalen = 2*4;
+ }
+
  data[0] = 0; // Version and reserved
  data[1] = ROAR_STREAM_PARA_FLAGS; // flags
- data[2] = reset == ROAR_RESET_FLAG ? ROAR_RESET_FLAG : ROAR_SET_FLAG;
- data[3] = flags;
+ data[2] = action;
+ data[3] = flags & 0x0000FFFF;
+
+ if ( flags & 0xFFFF0000 ) {
+  data[4] = (flags & 0xFFFF0000) >> 16;
+ }
 
  for (i = 0; i < m.datalen/2; i++) {
   data[i] = ROAR_HOST2NET16(data[i]);
@@ -644,6 +761,11 @@ int roar_stream_set_role  (struct roar_connection * con, struct roar_stream * s,
  uint16_t * data = (uint16_t *) m.data;
  int i;
 
+ if ( con == NULL || s == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
  memset(&m,  0, sizeof(m));
 
  m.cmd     = ROAR_CMD_SET_STREAM_PARA;
@@ -674,8 +796,10 @@ int roar_stream_s2m     (struct roar_stream * s, struct roar_message * m) {
  uint32_t * data;
  int i;
 
- if ( !(s && m) )
+ if ( s == NULL || m == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  m->datalen = _ROAR_STREAM_MESSAGE_LEN;
  data = (uint32_t*) m->data;
@@ -701,11 +825,15 @@ int roar_stream_m2s     (struct roar_stream * s, struct roar_message * m) {
  uint32_t * data;
  int i;
 
- if ( !(s && m) )
+ if ( s == NULL || m == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( m->datalen != _ROAR_STREAM_MESSAGE_LEN )
+ if ( m->datalen != _ROAR_STREAM_MESSAGE_LEN ) {
+  roar_err_set(ROAR_ERROR_MSGSIZE);
   return -1;
+ }
 
  s->pos = m->pos;
 
@@ -723,7 +851,7 @@ int roar_stream_m2s     (struct roar_stream * s, struct roar_message * m) {
  s->info.codec    = data[5];
 
  ROAR_DBG("roar_stream_m2s(*): s->info:");
- roar_debug_audio_info_print(&s->info);
+ roar_debug_audio_info_print(&(s->info));
 
  return 0;
 }
@@ -737,9 +865,9 @@ int roar_stream_m2s     (struct roar_stream * s, struct roar_message * m) {
                            "unknown")
 */
 
-struct {
+static struct {
  int    dir;
- char * name;
+ const char * name;
 } _libroar_dir[] = {
  {ROAR_DIR_PLAY,        "play"       },
  {ROAR_DIR_RECORD,      "record"     },
@@ -764,7 +892,7 @@ struct {
  {-1,                   "unknown"    }
 };
 
-char * roar_dir2str (int dir) {
+const char * roar_dir2str (const int dir) {
  int i;
 
  for (i = 0; _libroar_dir[i].dir != -1; i++)
@@ -774,7 +902,7 @@ char * roar_dir2str (int dir) {
  return _libroar_dir[i].name;
 }
 
-int roar_str2dir (char * name) {
+int roar_str2dir (const char * name) {
  int i;
 
  for (i = 0; _libroar_dir[i].dir != -1; i++)
@@ -792,97 +920,112 @@ int roar_str2dir (char * name) {
                            "unknown" )
 */
 
-struct {
+static struct {
  int    codec;
- char * name;
+ const char * name;
+ const char * mime;
 } _libroar_codec[] = {
  // PCM:
- {ROAR_CODEC_PCM_S_LE,    "pcm_s_le"   },
- {ROAR_CODEC_PCM_S_BE,    "pcm_s_be"   },
- {ROAR_CODEC_PCM_S_PDP,   "pcm_s_pdp"  },
- {ROAR_CODEC_PCM_U_LE,    "pcm_u_le"   },
- {ROAR_CODEC_PCM_U_BE,    "pcm_u_be"   },
- {ROAR_CODEC_PCM_U_PDP,   "pcm_u_pdp"  },
- {ROAR_CODEC_DEFAULT,     "default"    }, // alias
- {ROAR_CODEC_DEFAULT,     "pcm"        }, // alias
- {ROAR_CODEC_DEFAULT,     "raw"        }, // alias
+ {ROAR_CODEC_PCM_S_LE,    "pcm_s_le",    NULL},
+ {ROAR_CODEC_PCM_S_BE,    "pcm_s_be",    NULL},
+ {ROAR_CODEC_PCM_S_PDP,   "pcm_s_pdp",   NULL},
+ {ROAR_CODEC_PCM_U_LE,    "pcm_u_le",    NULL},
+ {ROAR_CODEC_PCM_U_BE,    "pcm_u_be",    NULL},
+ {ROAR_CODEC_PCM_U_PDP,   "pcm_u_pdp",   NULL},
+ {ROAR_CODEC_DEFAULT,     "default",     NULL}, // alias
+ {ROAR_CODEC_DEFAULT,     "pcm",         NULL}, // alias
+ {ROAR_CODEC_DEFAULT,     "raw",         NULL}, // alias
+ {ROAR_CODEC_PCM_S,       "pcm_s",       NULL}, // alias
+ {ROAR_CODEC_PCM_U,       "pcm_u",       NULL}, // alias
 
  // MIDI:
- {ROAR_CODEC_MIDI_FILE,   "midi_file"  },
- {ROAR_CODEC_MIDI,        "midi"       },
- {ROAR_CODEC_ROARMIDI,    "roarmidi"   },
+ {ROAR_CODEC_MIDI_FILE,   "midi_file",   NULL},
+ {ROAR_CODEC_MIDI,        "midi",        NULL},
+ {ROAR_CODEC_ROARMIDI,    "roarmidi",    NULL},
 
  // XIPH:
- {ROAR_CODEC_OGG_VORBIS,  "ogg_vorbis" },
- {ROAR_CODEC_OGG_VORBIS,  "vorbis"     }, // alias
- {ROAR_CODEC_FLAC,        "flac"       },
- {ROAR_CODEC_OGG_SPEEX,   "ogg_speex"  },
- {ROAR_CODEC_OGG_SPEEX,   "speex"      }, // alias
- {ROAR_CODEC_OGG_FLAC,    "ogg_flac"   },
- {ROAR_CODEC_OGG_GENERAL, "ogg_general"},
- {ROAR_CODEC_OGG_CELT,    "ogg_celt"   },
- {ROAR_CODEC_OGG,         "ogg"        },
- {ROAR_CODEC_ROAR_CELT,   "roar_celt"  },
- {ROAR_CODEC_ROAR_SPEEX,  "roar_speex" },
+ {ROAR_CODEC_OGG_VORBIS,  "ogg_vorbis",  "application/ogg"},
+ {ROAR_CODEC_OGG_VORBIS,  "vorbis",      "application/ogg"}, // alias
+ {ROAR_CODEC_FLAC,        "flac",        "audio/x-flac"},
+ {ROAR_CODEC_OGG_SPEEX,   "ogg_speex",   "audio/ogg; codecs=speex"},
+ {ROAR_CODEC_OGG_SPEEX,   "speex",       "audio/ogg; codecs=speex"}, // alias
+ {ROAR_CODEC_OGG_FLAC,    "ogg_flac",    "audio/ogg; codecs=flac"},
+ {ROAR_CODEC_OGG_GENERAL, "ogg_general", "application/ogg"},
+ {ROAR_CODEC_OGG_CELT,    "ogg_celt",    "audio/ogg; codecs=celt"},
+ {ROAR_CODEC_OGG,         "ogg",         "application/ogg"},
+ {ROAR_CODEC_OGG_OPUS,    "ogg_opus",    NULL},
+ {ROAR_CODEC_ROAR_OPUS,   "roar_opus",   NULL},
+ {ROAR_CODEC_ROAR_CELT,   "roar_celt",   NULL},
+ {ROAR_CODEC_ROAR_SPEEX,  "roar_speex",  NULL},
 
  // RAUM:
- {ROAR_CODEC_RAUM,        "raum"       },
- {ROAR_CODEC_RAUM_VORBIS, "raum_vorbis"},
- {ROAR_CODEC_RAUM_FLAC,   "raum_flac"  },
+ {ROAR_CODEC_RAUM,        "raum",        NULL},
+ {ROAR_CODEC_RAUM_VORBIS, "raum_vorbis", NULL},
+ {ROAR_CODEC_RAUM_FLAC,   "raum_flac",   NULL},
 
  // RIFF/WAVE like:
- {ROAR_CODEC_RIFF_WAVE,   "riff_wave"  },
- {ROAR_CODEC_RIFF_WAVE,   "wave"       }, // alias
- {ROAR_CODEC_RIFF_WAVE,   "wav"        }, // alias
- {ROAR_CODEC_RIFX,        "rifx"       },
- {ROAR_CODEC_AU,          "au"         },
- {ROAR_CODEC_AIFF,        "aiff"       },
+ {ROAR_CODEC_RIFF_WAVE,   "riff_wave",   "audio/x-wav"},
+ {ROAR_CODEC_RIFF_WAVE,   "wave",        "audio/x-wav"}, // alias
+ {ROAR_CODEC_RIFF_WAVE,   "wav",         "audio/x-wav"}, // alias
+ {ROAR_CODEC_RIFX,        "rifx",        NULL},
+ {ROAR_CODEC_AU,          "au",          "audio/basic"},
+ {ROAR_CODEC_AIFF,        "aiff",        "audio/aiff"},
 
  //Log codecs:
- {ROAR_CODEC_ALAW,        "alaw"       },
- {ROAR_CODEC_MULAW,       "mulaw"      },
- {ROAR_CODEC_MULAW,       "ulaw"       }, // alias
+ {ROAR_CODEC_ALAW,        "alaw",        NULL},
+ {ROAR_CODEC_AUTLAW_LE,   "autlaw_le",   NULL},
+ {ROAR_CODEC_AUTLAW_BE,   "autlaw_be",   NULL},
+ {ROAR_CODEC_AUTLAW,      "autlaw",      NULL}, // alias
+ {ROAR_CODEC_MULAW,       "mulaw",       NULL},
+ {ROAR_CODEC_MULAW,       "ulaw",        NULL}, // alias
+ {ROAR_CODEC_MUUTLAW_LE,  "muutlaw_le",  NULL},
+ {ROAR_CODEC_MUUTLAW_BE,  "muutlaw_be",  NULL},
+ {ROAR_CODEC_MUUTLAW,     "muutlaw",     NULL}, // alias
 
  //GSM:
- {ROAR_CODEC_GSM,         "gsm"        },
- {ROAR_CODEC_GSM49,       "gsm49"      },
+ {ROAR_CODEC_GSM,         "gsm",         NULL},
+ {ROAR_CODEC_GSM49,       "gsm49",       NULL},
+
+ //SPC-700 Bit Rate Reduction of
+ //Super Nintendo Entertainment System (SNES)
+ {ROAR_CODEC_BRR,         "brr",         NULL},
 
  // Meta Codecs:
- {ROAR_CODEC_META_VCLT,     "meta_vclt"    },
- {ROAR_CODEC_META_RALT,     "meta_ralt"    },
- {ROAR_CODEC_META_RALB,     "meta_ralb"    },
- {ROAR_CODEC_META_RALB_LE,  "meta_ralb_le" },
- {ROAR_CODEC_META_RALB_BE,  "meta_ralb_be" },
- {ROAR_CODEC_META_RALB_PDP, "meta_ralb_pdp"},
+ {ROAR_CODEC_META_VCLT,     "meta_vclt",     NULL},
+ {ROAR_CODEC_META_RALT,     "meta_ralt",     NULL},
+ {ROAR_CODEC_META_RALB,     "meta_ralb",     NULL},
+ {ROAR_CODEC_META_RALB_LE,  "meta_ralb_le",  NULL},
+ {ROAR_CODEC_META_RALB_BE,  "meta_ralb_be",  NULL},
+ {ROAR_CODEC_META_RALB_PDP, "meta_ralb_pdp", NULL},
 
  // light control:
- {ROAR_CODEC_DMX512,      "dmx512"     },
- {ROAR_CODEC_ROARDMX,     "roardmx"    },
+ {ROAR_CODEC_DMX512,      "dmx512",      NULL},
+ {ROAR_CODEC_ROARDMX,     "roardmx",     NULL},
 
  // Radio Data and Transmitter Control System:
- {ROAR_CODEC_RDS,         "rds"        },
+ {ROAR_CODEC_RDS,         "rds",         NULL},
 
  // User specific:
- {ROAR_CODEC_USER0,       "user0"      },
- {ROAR_CODEC_USER1,       "user1"      },
- {ROAR_CODEC_USER2,       "user2"      },
- {ROAR_CODEC_USER3,       "user3"      },
- {ROAR_CODEC_USER4,       "user4"      },
- {ROAR_CODEC_USER5,       "user5"      },
- {ROAR_CODEC_USER6,       "user6"      },
- {ROAR_CODEC_USER7,       "user7"      },
- {ROAR_CODEC_USER8,       "user8"      },
- {ROAR_CODEC_USER9,       "user9"      },
- {ROAR_CODEC_USER10,      "user10"     },
- {ROAR_CODEC_USER11,      "user11"     },
- {ROAR_CODEC_USER12,      "user12"     },
- {ROAR_CODEC_USER13,      "user13"     },
- {ROAR_CODEC_USER14,      "user14"     },
- {ROAR_CODEC_USER15,      "user15"     },
- {-1, NULL}
+ {ROAR_CODEC_USER0,       "user0",       NULL},
+ {ROAR_CODEC_USER1,       "user1",       NULL},
+ {ROAR_CODEC_USER2,       "user2",       NULL},
+ {ROAR_CODEC_USER3,       "user3",       NULL},
+ {ROAR_CODEC_USER4,       "user4",       NULL},
+ {ROAR_CODEC_USER5,       "user5",       NULL},
+ {ROAR_CODEC_USER6,       "user6",       NULL},
+ {ROAR_CODEC_USER7,       "user7",       NULL},
+ {ROAR_CODEC_USER8,       "user8",       NULL},
+ {ROAR_CODEC_USER9,       "user9",       NULL},
+ {ROAR_CODEC_USER10,      "user10",      NULL},
+ {ROAR_CODEC_USER11,      "user11",      NULL},
+ {ROAR_CODEC_USER12,      "user12",      NULL},
+ {ROAR_CODEC_USER13,      "user13",      NULL},
+ {ROAR_CODEC_USER14,      "user14",      NULL},
+ {ROAR_CODEC_USER15,      "user15",      NULL},
+ {-1, NULL, NULL}
 };
 
-int roar_str2codec(char * codec) {
+int roar_str2codec(const char * codec) {
  int i;
  int guess;
 
@@ -892,18 +1035,16 @@ int roar_str2codec(char * codec) {
  if ( (guess = atoi(codec)) > 0 )
   return guess;
 
- if ( codec == NULL || *codec == 0 )
-  return ROAR_CODEC_DEFAULT;
-
  for (i = 0; _libroar_codec[i].codec != -1; i++)
   if ( strcasecmp(_libroar_codec[i].name, codec) == 0 )
    return _libroar_codec[i].codec;
 
+ roar_err_set(ROAR_ERROR_NOENT);
  return -1;
 }
 
 
-char * roar_codec2str (int codec) {
+const char * roar_codec2str (const int codec) {
  int i;
 
  for (i = 0; _libroar_codec[i].codec != -1; i++)
@@ -913,7 +1054,101 @@ char * roar_codec2str (int codec) {
  return "unknown";
 }
 
-char * roar_streamstate2str(int streamstate) {
+int    roar_mime2codec (const char * mime) {
+ int i;
+
+ if ( mime == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ if ( *mime == 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return -1;
+ }
+
+ for (i = 0; _libroar_codec[i].codec != -1; i++)
+  if ( _libroar_codec[i].mime != NULL )
+   if ( strcasecmp(_libroar_codec[i].mime, mime) == 0 )
+    return _libroar_codec[i].codec;
+
+ roar_err_set(ROAR_ERROR_NOENT);
+ return -1;
+}
+
+const char * roar_codec2mime (const int    codec) {
+ int i;
+
+ for (i = 0; _libroar_codec[i].codec != -1; i++)
+  if ( _libroar_codec[i].codec == codec )
+   return _libroar_codec[i].mime;
+
+ roar_err_set(ROAR_ERROR_NOENT);
+ return NULL;
+}
+
+int32_t roar_str2rate(const char * rate) {
+ struct roar_audio_info info;
+ int ret;
+
+ if ( roar_profile2info(&info, rate) != -1 ) {
+  return info.rate;
+ }
+
+ ret = atoi(rate);
+
+ if ( ret == 0 && rate[0] != '0' ) {
+  roar_err_set(ROAR_ERROR_NOENT);
+  return -1;
+ }
+
+ return ret;
+}
+
+int32_t roar_str2bits(const char * bits) {
+ struct roar_audio_info info;
+ int ret;
+
+ if ( roar_profile2info(&info, bits) != -1 ) {
+  return info.bits;
+ }
+
+ ret = atoi(bits);
+
+ if ( ret == 0 && bits[0] != '0' ) {
+  roar_err_set(ROAR_ERROR_NOENT);
+  return -1;
+ }
+
+ return ret;
+}
+
+int32_t roar_str2channels(const char * channels) {
+ struct roar_audio_info info;
+ int ret;
+
+ if ( !strcasecmp(channels, "mono") ) {
+  return 1;
+ } else if ( !strcasecmp(channels, "stereo") ) {
+  return 2;
+ }
+
+ if ( roar_profile2info(&info, channels) != -1 ) {
+  return info.channels;
+ }
+
+ ret = atoi(channels);
+
+ if ( ret == 0 && channels[0] != '0' ) {
+  roar_err_set(ROAR_ERROR_NOENT);
+  return -1;
+ }
+
+ return ret;
+}
+
+
+const char * roar_streamstate2str(int streamstate) {
  switch (streamstate) {
   case ROAR_STREAMSTATE_UNUSED:  return "unused";  break;
   case ROAR_STREAMSTATE_INITING: return "initing"; break;
@@ -925,9 +1160,9 @@ char * roar_streamstate2str(int streamstate) {
  return "unknown";
 }
 
-struct {
+static struct {
  int    role;
- char * name;
+ const char * name;
 } _libroar_role[] = {
  {ROAR_ROLE_UNKNOWN,          "unknown"         },
  {ROAR_ROLE_NONE,             "none"            },
@@ -947,17 +1182,20 @@ struct {
  {-1, NULL}
 };
 
-int    roar_str2role  (char * role) {
+int    roar_str2role  (const char * role) {
  int i;
 
+ roar_err_clear();
+
  for (i = 0; _libroar_role[i].name != NULL; i++)
   if ( !strcasecmp(_libroar_role[i].name, role) )
    return _libroar_role[i].role;
 
+ roar_err_set(ROAR_ERROR_NOENT);
  return ROAR_ROLE_UNKNOWN;
 }
 
-char * roar_role2str  (int    role) {
+const char * roar_role2str  (const int    role) {
  int i;
 
  for (i = 0; _libroar_role[i].name != NULL; i++)
@@ -968,8 +1206,10 @@ char * roar_role2str  (int    role) {
 }
 
 ssize_t roar_info2samplesize (struct roar_audio_info * info) {
- if ( info == NULL )
+ if ( info == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  switch (info->codec) {
   case ROAR_CODEC_PCM_S_LE:
@@ -991,6 +1231,7 @@ ssize_t roar_info2samplesize (struct roar_audio_info * info) {
     return 26;
    break;
   default:
+    roar_err_set(ROAR_ERROR_INVAL);
     return -1;
    break;
  }
@@ -1026,6 +1267,11 @@ static struct {
                 .bits     = ROAR_BITS_DEFAULT,
                 .channels = ROAR_CHANNELS_DEFAULT,
                 .codec    = ROAR_CODEC_DEFAULT}},
+ {"default-server",
+               {.rate     = ROAR_RATE_DEFAULT,
+                .bits     = ROAR_ROARD_BITS,
+                .channels = ROAR_CHANNELS_DEFAULT,
+                .codec    = ROAR_CODEC_DEFAULT}},
  {"wav",       {.rate     = ROAR_RATE_DEFAULT,
                 .bits     = ROAR_BITS_DEFAULT,
                 .channels = ROAR_CHANNELS_DEFAULT,
@@ -1042,7 +1288,10 @@ static struct {
  {"speex-nb",  {.rate =   8000, .bits = 16, .channels =  2, .codec = ROAR_CODEC_ROAR_SPEEX}},
  {"speex-wb",  {.rate =  16000, .bits = 16, .channels =  2, .codec = ROAR_CODEC_ROAR_SPEEX}},
  {"speex-uwb", {.rate =  32000, .bits = 16, .channels =  2, .codec = ROAR_CODEC_ROAR_SPEEX}},
+ {"opus",      {.rate =  48000, .bits = 16, .channels =  2, .codec = ROAR_CODEC_ROAR_OPUS}},
  {"gsm",       {.rate =   8000, .bits = 16, .channels =  1, .codec = ROAR_CODEC_GSM}},
+ {"brr",       {.rate =   8000, .bits = 32, .channels =  1, .codec = ROAR_CODEC_BRR}},
+ {"brr6k",     {.rate =   6000, .bits = 32, .channels =  1, .codec = ROAR_CODEC_BRR}},
  {"rds",       {.rate =      0, .bits =  0, .channels =  0, .codec = ROAR_CODEC_RDS}},
  {"midi",      {.rate =      0, .bits =  8, .channels = 16, .codec = ROAR_CODEC_MIDI}},
  {"dmx512",    {.rate =      0, .bits =  8, .channels =  0, .codec = ROAR_CODEC_DMX512}},
@@ -1052,6 +1301,11 @@ static struct {
 int     roar_profile2info    (struct roar_audio_info * info, const char * profile) {
  int i;
 
+ if ( info == NULL || profile == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
  for (i = 0; _libroar_aiprofiles[i].name != NULL; i++) {
   if ( !strcasecmp(_libroar_aiprofiles[i].name, profile) ) {
    memcpy(info, &(_libroar_aiprofiles[i].info), sizeof(struct roar_audio_info));
@@ -1059,6 +1313,7 @@ int     roar_profile2info    (struct roar_audio_info * info, const char * profil
   }
  }
 
+ roar_err_set(ROAR_ERROR_NOENT);
  return -1;
 }
 
@@ -1066,8 +1321,10 @@ ssize_t   roar_profiles_list   (const char ** list, size_t len, size_t offset) {
  size_t i;
  int idx = 0;
 
- if ( list == NULL )
+ if ( list == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  if ( len == 0 )
   return 0;
