@@ -1,7 +1,7 @@
 //vio.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of libroar a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -33,6 +33,7 @@
  *  them with any software that uses libesd, libartsc or libpulse*.
  */
 
+#define _LIBROAR_NOATTR_TO_STATIC /* ignore warnings for TO_STATIC functions */
 #include "libroar.h"
 
 #ifdef ROAR_HAVE_H_SYS_IOCTL
@@ -44,9 +45,13 @@
 #endif
 
 int roar_vio_init_calls (struct roar_vio_calls * calls) {
+ roar_debug_warn_obsolete("roar_vio_init_calls", "roar_vio_clear_calls", NULL);
+
 #ifdef _CAN_OPERATE
- if ( calls == NULL )
+ if ( calls == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  memset((void*)calls, 0, sizeof(struct roar_vio_calls));
 
@@ -70,9 +75,22 @@ int roar_vio_init_calls (struct roar_vio_calls * calls) {
 #endif
 }
 
-int roar_vio_set_inst (struct roar_vio_calls * vio, void * inst) {
- if ( vio == NULL )
+int roar_vio_clear_calls (struct roar_vio_calls * calls) {
+ if ( calls == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
+
+ memset((void*)calls, 0, sizeof(struct roar_vio_calls));
+
+ return 0;
+}
+
+int roar_vio_set_inst (struct roar_vio_calls * vio, void * inst) {
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
 
  vio->inst = inst;
 
@@ -84,108 +102,204 @@ int roar_vio_set_fh   (struct roar_vio_calls * vio, int fh) {
 }
 
 int roar_vio_get_fh   (struct roar_vio_calls * vio) {
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return ((int)(ROAR_INSTINT)vio->inst) - 1;
 }
 
 
 ssize_t roar_vio_read (struct roar_vio_calls * vio, void *buf, size_t count) {
+ ssize_t ret;
+
  ROAR_DBG("roar_vio_read(vio=%p, buf=%p, count=%u) = ?", vio, buf, (unsigned int)count);
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->read == NULL )
+ if ( vio->read == NULL ) {
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
+ }
 
- return vio->read(vio, buf, count);
+ roar_err_clear_all();
+ ret = vio->read(vio, buf, count);
+ roar_err_update();
+
+ return ret;
 }
 
 ssize_t roar_vio_write(struct roar_vio_calls * vio, void *buf, size_t count) {
+ ssize_t ret;
+
  ROAR_DBG("roar_vio_write(vio=%p, buf=%p, count=%u) = ?", vio, buf, (unsigned int)count);
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->write == NULL )
+ if ( vio->write == NULL ) {
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
+ }
 
- return vio->write(vio, buf, count);
+ roar_err_clear_all();
+ ret = vio->write(vio, buf, count);
+ roar_err_update();
+
+ return ret;
 }
 
 off_t   roar_vio_lseek(struct roar_vio_calls * vio, off_t offset, int whence) {
+ off_t ret;
+
  ROAR_DBG("roar_vio_lseek(vio=%p, offset=%u, whence=%i) = ?", vio, (unsigned int)offset, whence);
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->lseek == NULL )
+ if ( vio->lseek == NULL ) {
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
+ }
 
- return vio->lseek(vio, offset, whence);
+ roar_err_clear_all();
+ ret = vio->lseek(vio, offset, whence);
+ roar_err_update();
+
+ return ret;
 }
 
 int     roar_vio_nonblock(struct roar_vio_calls * vio, int state) {
+ int ret;
+
  ROAR_DBG("roar_vio_nonblock(vio=%p, state=%i) = ?", vio, state);
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->nonblock == NULL )
+ if ( vio->nonblock == NULL ) {
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
+ }
 
- return vio->nonblock(vio, state);
+ roar_err_clear_all();
+ ret = vio->nonblock(vio, state);
+ roar_err_update();
+
+ return ret;
 }
 
 int     roar_vio_sync    (struct roar_vio_calls * vio) {
+ int ret;
+
  ROAR_DBG("roar_vio_sync(vio=%p) = ?", vio);
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->sync == NULL )
+ if ( vio->sync == NULL ) {
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
+ }
 
- return vio->sync(vio);
+ roar_err_clear_all();
+ ret = vio->sync(vio);
+ roar_err_update();
+
+ return ret;
 }
 
 int     roar_vio_ctl     (struct roar_vio_calls * vio, int cmd, void * data) {
+ int ret;
+
  ROAR_DBG("roar_vio_ctl(vio=%p, cmd=%i, data=%p) = ?", vio, cmd, data);
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  ROAR_DBG("roar_vio_ctl(vio=%p, cmd=0x%.8x, data=%p): vio->ctl=%p", vio, cmd, data, vio->ctl);
 
- if ( vio->ctl == NULL )
-  return -1;
+ switch (cmd) {
+  case ROAR_VIO_CTL_CONFLICTING_ID_0:
+  case ROAR_VIO_CTL_CONFLICTING_ID_1:
+    ROAR_ERR("roar_vio_ctl(vio=%p, cmd=0x%.8x, data=%p): Your progam uses a VIO ctl call with a conflicting ID.", vio, cmd, data);
+    ROAR_ERR("roar_vio_ctl(vio=%p, cmd=0x%.8x, data=%p): Please recompile your program to fix this. (No additional steps are required beside that.)", vio, cmd, data);
+    ROAR_DBG("roar_vio_ctl(vio=%p, cmd=0x%.8x, data=%p) = -1", vio, cmd, data);
+    roar_err_set(ROAR_ERROR_BADRQC);
+    return -1;
+   break;
+ }
 
- return vio->ctl(vio, cmd, data);
+ if ( vio->ctl == NULL ) {
+  roar_err_set(ROAR_ERROR_NOSYS);
+  return -1;
+ }
+
+ roar_err_clear_all();
+ ret = vio->ctl(vio, cmd, data);
+ roar_err_update();
+
+ return ret;
 }
 
 int     roar_vio_close    (struct roar_vio_calls * vio) {
+ int ret;
+
  ROAR_DBG("roar_vio_close(vio=%p) = ?", vio);
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->close == NULL )
+ if ( vio->close == NULL ) {
+  roar_err_set(ROAR_ERROR_NOSYS);
   return -1;
+ }
 
- return vio->close(vio);
+ roar_err_clear_all();
+ ret = vio->close(vio);
+ roar_err_update();
+
+ return ret;
 }
 
 // specal commands:
 int     roar_vio_accept  (struct roar_vio_calls * calls, struct roar_vio_calls * dst) {
- if (dst == NULL || calls == NULL)
+ if (dst == NULL || calls == NULL) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return roar_vio_ctl(dst, ROAR_VIO_CTL_ACCEPT, calls);
 }
 
 int     roar_vio_shutdown(struct roar_vio_calls * vio,   int how) {
+ if (vio == NULL) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ if ( ( (how | ROAR_VIO_SHUTDOWN_READ|ROAR_VIO_SHUTDOWN_WRITE|ROAR_VIO_SHUTDOWN_LISTEN) -
+        (ROAR_VIO_SHUTDOWN_READ|ROAR_VIO_SHUTDOWN_WRITE|ROAR_VIO_SHUTDOWN_LISTEN) ) != 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return -1;
+ }
+
  return roar_vio_ctl(vio, ROAR_VIO_CTL_SHUTDOWN, &how);
 }
 
@@ -194,21 +308,32 @@ int     roar_vio_open_file     (struct roar_vio_calls * calls, char * filename,
 #ifdef _CAN_OPERATE
  int fh;
 
- if ( calls == NULL || filename == NULL )
+ roar_debug_warn_obsolete("roar_vio_open_file", "roar_vio_open_dstr", NULL);
+
+ if ( calls == NULL || filename == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
 #ifdef ROAR_TARGET_WIN32
  flags |= O_BINARY;
 #endif
 
- if ( (fh = open(filename, flags, mode)) == -1 )
-  return -1;
-
- if ( roar_vio_open_fh(calls, fh) == -1 ) {
-  close(fh);
+ roar_err_clear_all();
+ if ( (fh = open(filename, flags, mode)) == -1 ) {
+  ROAR_DBG("roar_vio_open_file(*): errno=%s", strerror(errno));
+  roar_err_update();
+  ROAR_DBG("roar_vio_open_file(*): errno=%s", strerror(errno));
   return -1;
  }
 
+ if ( roar_vio_open_fh(calls, fh) == -1 ) {
+  close(fh);
+  roar_err_update();
+  return -1;
+ }
+
+ roar_err_update();
  return 0;
 #else
  return -1;
@@ -216,18 +341,26 @@ int     roar_vio_open_file     (struct roar_vio_calls * calls, char * filename,
 }
 
 int     roar_vio_open_fh       (struct roar_vio_calls * calls, int fh) {
- if ( calls == NULL )
+ if ( calls == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( roar_vio_init_calls(calls) == -1 )
+ roar_libroar_nowarn();
+ if ( roar_vio_init_calls(calls) == -1 ) {
+  roar_libroar_warn();
   return -1;
+ }
+ roar_libroar_warn();
 
  return roar_vio_set_fh(calls, fh);
 }
 
 int     roar_vio_open_fh_socket(struct roar_vio_calls * calls, int fh) {
- if ( calls == NULL )
+ if ( calls == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  if ( roar_vio_open_fh(calls, fh) == -1 )
   return -1;
@@ -249,8 +382,10 @@ int     roar_vio_open_fh_socket(struct roar_vio_calls * calls, int fh) {
 int     roar_vio_open_socket   (struct roar_vio_calls * calls, char * host, int port) {
  int fh;
 
- if ( calls == NULL )
+ if ( calls == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  if ( (fh = roar_socket_connect(host, port)) == -1 )
   return -1;
@@ -261,8 +396,10 @@ int     roar_vio_open_socket   (struct roar_vio_calls * calls, char * host, int
 int     roar_vio_open_socket_listen(struct roar_vio_calls * calls, int type, char * host, int port) {
  int fh;
 
- if ( calls == NULL )
+ if ( calls == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  if ( (fh = roar_socket_listen(type, host, port)) == -1 )
   return -1;
@@ -270,45 +407,6 @@ int     roar_vio_open_socket_listen(struct roar_vio_calls * calls, int type, cha
  return roar_vio_open_fh_socket(calls, fh);
 }
 
-int     roar_vio_simple_stream (struct roar_vio_calls * calls, int rate, int channels, int bits, int codec,
-                                                               char * server, int dir, char * name) {
- int fh;
-
- if ( calls == NULL )
-  return -1;
-
- roar_libroar_nowarn();
- if ( (fh = roar_simple_stream(rate, channels, bits, codec, server, dir, name)) == -1 ) {
-  roar_libroar_warn();
-  return -1;
- }
- roar_libroar_warn();
-
- return roar_vio_open_fh_socket(calls, fh);
-}
-
-int     roar_vio_simple_new_stream_obj (struct roar_vio_calls * calls,
-                                        struct roar_connection * con,
-                                        struct roar_stream * s,
-                                        int rate, int channels, int bits, int codec, int dir) {
- struct roar_stream stream;
- int fh;
-
- if ( calls == NULL )
-  return -1;
-
- if ( s == NULL )
-  s = &stream;
-
- roar_libroar_nowarn();
- if ( (fh = roar_simple_new_stream_obj(con, s, rate, channels, bits, codec, dir)) == -1 ) {
-  roar_libroar_warn();
-  return -1;
- }
- roar_libroar_warn();
-
- return roar_vio_open_fh_socket(calls, fh);
-}
 
 // VIOs:
 
@@ -317,6 +415,7 @@ ssize_t roar_vio_basic_read (struct roar_vio_calls * vio, void *buf, size_t coun
 #ifdef _CAN_OPERATE
  return read(roar_vio_get_fh(vio), buf, count);
 #else
+ roar_err_set(ROAR_ERROR_NOSYS);
  return -1;
 #endif
 }
@@ -325,6 +424,7 @@ ssize_t roar_vio_basic_write(struct roar_vio_calls * vio, void *buf, size_t coun
 #ifdef _CAN_OPERATE
  return write(roar_vio_get_fh(vio), buf, count);
 #else
+ roar_err_set(ROAR_ERROR_NOSYS);
  return -1;
 #endif
 }
@@ -333,6 +433,7 @@ off_t   roar_vio_basic_lseek(struct roar_vio_calls * vio, off_t offset, int when
 #ifdef _CAN_OPERATE
  return lseek(roar_vio_get_fh(vio), offset, whence);
 #else
+ roar_err_set(ROAR_ERROR_NOSYS);
  return -1;
 #endif
 }
@@ -360,19 +461,53 @@ int     roar_vio_basic_sync    (struct roar_vio_calls * vio) {
 int     roar_vio_basic_ctl     (struct roar_vio_calls * vio, int cmd, void * data) {
 #ifdef ROAR_HAVE_H_SYS_IOCTL
  struct roar_vio_sysio_ioctl * sysioctl;
+#endif
+#if defined(ROAR_HAVE_GETSOCKOPT) || defined(ROAR_HAVE_SETSOCKOPT)
+ struct roar_vio_sysio_sockopt  * syssockopt;
 #endif
  int tmp;
  int s_r = 0, s_w = 0;
+#if defined(ROAR_HAVE_GETSOCKNAME) || defined(ROAR_HAVE_GETPEERNAME)
+ union {
+  struct sockaddr     sa;
+#if defined(ROAR_HAVE_IPV4) || defined(ROAR_HAVE_IPV6)
+  struct sockaddr_in  in;
+#endif
+#ifdef ROAR_HAVE_UNIX
+  struct sockaddr_un  un;
+#endif
+#ifdef ROAR_HAVE_LIBDNET
+  struct sockaddr_dn  dn;
+#endif
+#ifdef ROAR_HAVE_IPV6
+  struct sockaddr_in6 in6;
+#endif
+#ifdef ROAR_HAVE_IPX
+  struct sockaddr_ipx ipx;
+#endif
+ } sockaddr;
+ socklen_t socklen;
+ struct roar_sockname * rsockname;
+#endif
 
- if ( vio == NULL || cmd == -1 )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
+
+ if ( cmd == -1 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return -1;
+ }
 
  ROAR_DBG("roar_vio_basic_ctl(vio=%p, cmd=0x%.8x, data=%p) = ?", vio, cmd, data);
 
  switch (cmd) {
   case ROAR_VIO_CTL_GET_NAME:
-    if ( data == NULL )
+    if ( data == NULL ) {
+     roar_err_set(ROAR_ERROR_FAULT);
      return -1;
+    }
 
     *(char**)data = "basic";
     return 0;
@@ -421,8 +556,10 @@ int     roar_vio_basic_ctl     (struct roar_vio_calls * vio, int cmd, void * dat
      tmp -= ROAR_VIO_SHUTDOWN_WRITE;
     }
 
-    if ( tmp != 0 ) /* we currently only support R and W shutdowns */
+    if ( tmp != 0 ) { /* we currently only support R and W shutdowns */
+     roar_err_set(ROAR_ERROR_NOTSUP);
      return -1;
+    }
 
     if ( s_r && s_w ) {
      tmp = SHUT_RDWR;
@@ -436,14 +573,124 @@ int     roar_vio_basic_ctl     (struct roar_vio_calls * vio, int cmd, void * dat
 
     return ROAR_SHUTDOWN(roar_vio_get_fh(vio), tmp);
    break;
+#if defined(ROAR_HAVE_GETSOCKNAME) || defined(ROAR_HAVE_GETPEERNAME)
+  case ROAR_VIO_CTL_GET_SOCKNAME:
+  case ROAR_VIO_CTL_GET_PEERNAME:
+    if ( data == NULL ) {
+     roar_err_set(ROAR_ERROR_FAULT);
+     return -1;
+    }
+
+    rsockname = data;
+
+    socklen = sizeof(sockaddr);
+
+    if ( cmd == ROAR_VIO_CTL_GET_SOCKNAME ) {
+#ifdef ROAR_HAVE_GETSOCKNAME
+     tmp = getsockname(roar_vio_get_fh(vio), &(sockaddr.sa), &socklen);
+#else
+     roar_err_set(ROAR_ERROR_NOSYS);
+     return -1;
+#endif
+    } else if ( cmd == ROAR_VIO_CTL_GET_PEERNAME ) {
+#ifdef ROAR_HAVE_GETPEERNAME
+     tmp = getpeername(roar_vio_get_fh(vio), &(sockaddr.sa), &socklen);
+#else
+     roar_err_set(ROAR_ERROR_NOSYS);
+     return -1;
+#endif
+    } else {
+     // memory corruption:
+     roar_panic(ROAR_FATAL_ERROR_MEMORY_CORRUPTION, NULL);
+     roar_err_set(ROAR_ERROR_CHERNOBYL);
+     return -1;
+    }
+
+    if ( tmp == -1 )
+     return -1;
+
+    memset(rsockname, 0, sizeof(struct roar_sockname));
+
+    switch (sockaddr.sa.sa_family) {
+#if defined(AF_UNIX) && defined(ROAR_HAVE_UNIX)
+     case AF_UNIX:
+       rsockname->type = ROAR_SOCKET_TYPE_UNIX;
+       if ( sockaddr.un.sun_path[0] == 0 ) {
+        rsockname->addr = roar_mm_malloc(sizeof(sockaddr.un.sun_path));
+        if ( rsockname->addr == NULL )
+         return -1;
+        memcpy(rsockname->addr, sockaddr.un.sun_path, sizeof(sockaddr.un.sun_path));
+       } else {
+        rsockname->addr = roar_mm_strdup(sockaddr.un.sun_path);
+       }
+      break;
+#endif
+#if defined(AF_DECnet) && defined(ROAR_HAVE_LIBDNET)
+     case AF_DECnet:
+       rsockname->type = ROAR_SOCKET_TYPE_DECNET;
+
+       if ( sockaddr.dn.sdn_add.a_len != 2 ) {
+        roar_err_set(ROAR_ERROR_NOTSUP);
+        return -1;
+       }
+
+       rsockname->addr = roar_mm_malloc(28);
+       if ( rsockname->addr == NULL )
+        return -1;
+
+       snprintf(rsockname->addr, 28, "%i.%i::",
+                 sockaddr.dn.sdn_add.a_addr[1] >> 2,
+                 sockaddr.dn.sdn_add.a_addr[0] + ((sockaddr.dn.sdn_add.a_addr[1] & 0x03) << 8));
+
+       rsockname->port = sockaddr.dn.sdn_objnum;
+       if ( sockaddr.dn.sdn_objnum == 0 ) {
+        tmp = strlen(rsockname->addr);
+        memcpy(rsockname->addr + tmp, sockaddr.dn.sdn_objname, sockaddr.dn.sdn_objnamel);
+        rsockname->addr[tmp + sockaddr.dn.sdn_objnamel] = 0;
+       }
+      break;
+#endif
+#if defined(AF_INET) && (defined(ROAR_HAVE_IPV4) || defined(ROAR_HAVE_IPV6))
+     case AF_INET:
+       rsockname->type = ROAR_SOCKET_TYPE_INET;
+       rsockname->port = ntohs(sockaddr.in.sin_port);
+       rsockname->addr = roar_mm_strdup(inet_ntoa(sockaddr.in.sin_addr));
+      break;
+#endif
+#if defined(AF_INET6) && defined(ROAR_HAVE_IPV6)
+     case AF_INET6:
+       rsockname->type = ROAR_SOCKET_TYPE_INET6;
+       rsockname->port = ntohs(sockaddr.in6.sin6_port);
+      break;
+#endif
+     default:
+       roar_err_set(ROAR_ERROR_NOTSUP);
+       return -1;
+    }
+    return 0;
+#endif
+   break;
 #ifdef ROAR_HAVE_H_SYS_IOCTL
   case ROAR_VIO_CTL_SYSIO_IOCTL:
     sysioctl = data;
     return ioctl(roar_vio_get_fh(vio), sysioctl->cmd, sysioctl->argp);
    break;
+#endif
+#ifdef ROAR_HAVE_GETSOCKOPT
+  case ROAR_VIO_CTL_GET_SYSIO_SOCKOPT:
+    syssockopt = data;
+    return getsockopt(roar_vio_get_fh(vio), syssockopt->level, syssockopt->optname, syssockopt->optval, &(syssockopt->optlen));
+   break;
+#endif
+#ifdef ROAR_HAVE_SETSOCKOPT
+  case ROAR_VIO_CTL_SET_SYSIO_SOCKOPT:
+    syssockopt = data;
+    return setsockopt(roar_vio_get_fh(vio), syssockopt->level, syssockopt->optname, syssockopt->optval, syssockopt->optlen);
+   break;
 #endif
  }
 
+ roar_err_set(ROAR_ERROR_BADRQC);
  return -1;
 }
 
@@ -454,14 +701,17 @@ int     roar_vio_basic_close    (struct roar_vio_calls * vio) {
 
  return 0;
 #else
+ roar_err_set(ROAR_ERROR_NOSYS);
  return -1;
 #endif
 }
 
 // null
 ssize_t roar_vio_null_rw    (struct roar_vio_calls * vio, void *buf, size_t count) {
- if ( vio == NULL || buf == NULL )
+ if ( vio == NULL || buf == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  return 0;
 }
@@ -472,8 +722,10 @@ int     roar_vio_null_sync    (struct roar_vio_calls * vio) {
 
 // pass
 int     roar_vio_open_pass    (struct roar_vio_calls * calls, struct roar_vio_calls * dst) {
- if ( calls == NULL )
+ if ( calls == NULL || dst == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
  memset((void*)calls, 0, sizeof(struct roar_vio_calls));
 
@@ -511,17 +763,31 @@ int     roar_vio_pass_sync    (struct roar_vio_calls * vio) {
 }
 
 int     roar_vio_pass_ctl     (struct roar_vio_calls * vio, int cmd, void * data) {
- if (vio == NULL || cmd == -1)
+ if (vio == NULL) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
+
+ if (cmd == -1) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return -1;
+ }
 
  ROAR_DBG("roar_vio_pass_ctl(vio=%p, cmd=0x%.8x, data=%p) = ?", vio, cmd, data);
 
  switch (cmd) {
   case ROAR_VIO_CTL_GET_NAME:
-    if ( data == NULL )
+    if ( data == NULL ) {
+     roar_err_set(ROAR_ERROR_FAULT);
      return -1;
+    }
 
-    *(char**)data = "pass";
+    // dirty trick to get real name...
+    if ( vio->read == roar_vio_re_read ) {
+     *(char**)data = "re";
+    } else {
+     *(char**)data = "pass";
+    }
     return 0;
    break;
   case ROAR_VIO_CTL_GET_NEXT:
@@ -557,13 +823,17 @@ ssize_t roar_vio_re_read (struct roar_vio_calls * vio, void *buf, size_t count)
   size_t len =  0;
  ssize_t r   = -1;
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->inst == NULL )
+ if ( vio->inst == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- errno = 0;
+ roar_err_clear_all();
 
  while ( (r = roar_vio_read((struct roar_vio_calls *) vio->inst, buf, count)) > 0 ) {
   len   += r;
@@ -583,13 +853,17 @@ ssize_t roar_vio_re_write(struct roar_vio_calls * vio, void *buf, size_t count)
   size_t len =  0;
  ssize_t r   = -1;
 
- if ( vio == NULL )
+ if ( vio == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- if ( vio->inst == NULL )
+ if ( vio->inst == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return -1;
+ }
 
- errno = 0;
+ roar_err_clear_all();
 
  while ( (r = roar_vio_write((struct roar_vio_calls *) vio->inst, buf, count)) > 0 ) {
   len   += r;
