@@ -1,7 +1,7 @@
 //vio_dstr.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2011
  *
  *  This file is part of libroar a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -70,6 +70,16 @@ grep '^#define ROAR_VIO_DSTR_OBJT_' vio_dstr.h | cut -d' ' -f2 | while read objt
  {ROAR_VIO_DSTR_OBJT_EXEC,       "exec",
       NULL, NULL,
       {ROAR_VIO_DEF_TYPE_EOL}},
+/* special devices */
+ {ROAR_VIO_DSTR_OBJT_NULL,       "null",
+      NULL, NULL,
+      {ROAR_VIO_DEF_TYPE_EOL}},
+ {ROAR_VIO_DSTR_OBJT_ZERO,       "zero",
+      NULL, NULL,
+      {ROAR_VIO_DEF_TYPE_EOL}},
+ {ROAR_VIO_DSTR_OBJT_FULL,       "full",
+      NULL, NULL,
+      {ROAR_VIO_DEF_TYPE_EOL}},
 
  {ROAR_VIO_DSTR_OBJT_SOCKET,     "socket",
       NULL, NULL,
@@ -143,6 +153,9 @@ grep '^#define ROAR_VIO_DSTR_OBJT_' vio_dstr.h | cut -d' ' -f2 | while read objt
  {ROAR_VIO_DSTR_OBJT_GZIP,       "gzip",
       NULL, NULL,
       {ROAR_VIO_DEF_TYPE_EOL}},
+ {ROAR_VIO_DSTR_OBJT_ZLIB,       "zlib",
+      NULL, NULL,
+      {ROAR_VIO_DEF_TYPE_EOL}},
  {ROAR_VIO_DSTR_OBJT_BZIP2,      "bzip2",
       NULL, NULL,
       {ROAR_VIO_DEF_TYPE_EOL}},
@@ -171,6 +184,16 @@ grep '^#define ROAR_VIO_DSTR_OBJT_' vio_dstr.h | cut -d' ' -f2 | while read objt
  {ROAR_VIO_DSTR_OBJT_SSLTLS,     "ssltls",
       NULL, NULL,
       {ROAR_VIO_DEF_TYPE_EOL}},
+/* Random numbers */
+ {ROAR_VIO_DSTR_OBJT_NRANDOM,    "nrandom",
+      NULL, NULL,
+      {ROAR_VIO_DEF_TYPE_EOL}},
+ {ROAR_VIO_DSTR_OBJT_URANDOM,    "urandom",
+      NULL, NULL,
+      {ROAR_VIO_DEF_TYPE_EOL}},
+ {ROAR_VIO_DSTR_OBJT_SRANDOM,    "srandom",
+      NULL, NULL,
+      {ROAR_VIO_DEF_TYPE_EOL}},
 
  {ROAR_VIO_DSTR_OBJT_TRANSCODE,  "transcode", /* TODO  */
       NULL, NULL,
@@ -308,8 +331,13 @@ int     roar_vio_open_default (struct roar_vio_calls * calls, struct roar_vio_de
   case ROAR_VIO_DEF_TYPE_NONE:
    break;
   case ROAR_VIO_DEF_TYPE_FILE:
-    if ( roar_vio_open_file(calls, def->d.file, def->o_flags, def->o_mode) == -1 )
+    roar_libroar_nowarn();
+    if ( roar_vio_open_file(calls, def->d.file, def->o_flags, def->o_mode) == -1 ) {
+     roar_libroar_warn();
+     ROAR_DBG("roar_vio_open_default(*): Can not open file: %i", roar_error);
      return -1;
+    }
+    roar_libroar_warn();
    break;
   case ROAR_VIO_DEF_TYPE_SOCKET:
      if ( roar_vio_open_def_socket(calls, def, opts) == -1 )
@@ -335,7 +363,7 @@ int     roar_vio_open_dstr    (struct roar_vio_calls * calls, char * dstr, struc
  return roar_vio_open_dstr_vio(calls, dstr, def, dnum, NULL);
 }
 
-#define _ret(x) roar_mm_free(dstr); return (x)
+#define _ret(x) do { int _err = roar_error; roar_mm_free(dstr); roar_error = _err; roar_err_to_errno(); return (x); } while (0)
 
 int     roar_vio_open_dstr_vio(struct roar_vio_calls * calls,
                                char * dstr, struct roar_vio_defaults * def, int dnum,
@@ -511,6 +539,7 @@ int     roar_vio_dstr_set_defaults(struct roar_vio_dstr_chain * chain, int len,
    case ROAR_VIO_DSTR_OBJT_RE:
    case ROAR_VIO_DSTR_OBJT_RTP2: // we currently only forward the defs
    case ROAR_VIO_DSTR_OBJT_GZIP:
+   case ROAR_VIO_DSTR_OBJT_ZLIB:
    case ROAR_VIO_DSTR_OBJT_BZIP2:
    case ROAR_VIO_DSTR_OBJT_PGP:
    case ROAR_VIO_DSTR_OBJT_PGP_ENC:
@@ -525,6 +554,10 @@ int     roar_vio_dstr_set_defaults(struct roar_vio_dstr_chain * chain, int len,
 
      next->def = c->def;
     break;
+   case ROAR_VIO_DSTR_OBJT_NULL:
+   case ROAR_VIO_DSTR_OBJT_ZERO:
+   case ROAR_VIO_DSTR_OBJT_FULL:
+   case ROAR_VIO_DSTR_OBJT_NRANDOM:
    case ROAR_VIO_DSTR_OBJT_TANTALOS:
      next->def = &(next->store_def);
      roar_vio_dstr_init_defaults_c(next->def, ROAR_VIO_DEF_TYPE_NONE, NULL, -1);
@@ -721,7 +754,7 @@ int     roar_vio_dstr_set_defaults(struct roar_vio_dstr_chain * chain, int len,
  return 0;
 }
 
-#define _ret(x) roar_vio_close(calls); ROAR_DBG("roar_vio_dstr_build_chain(*) = %i", (x)); return (x)
+#define _ret(x) do { int _err = roar_error; roar_vio_close(calls); roar_error = _err; roar_err_to_errno(); ROAR_DBG("roar_vio_dstr_build_chain(*) = %i", (x)); return (x); } while(0)
 
 int     roar_vio_dstr_build_chain(struct roar_vio_dstr_chain * chain, struct roar_vio_calls * calls,
                                   struct roar_vio_calls * vio) {
@@ -736,18 +769,18 @@ int     roar_vio_dstr_build_chain(struct roar_vio_dstr_chain * chain, struct roa
  if ( chain == NULL || calls == NULL )
   return -1;
 
- if ( roar_vio_open_stack(calls) == -1 )
+ if ( roar_vio_open_stack2(calls, NULL) == -1 )
   return -1;
 
  ROAR_DBG("roar_vio_dstr_build_chain(*): chain=%p", chain);
 
  if ( (def = chain->def) != NULL ) {
-  if ( (tc = malloc(sizeof(struct roar_vio_calls))) == NULL ) {
+  if ( (tc = roar_mm_malloc(sizeof(struct roar_vio_calls))) == NULL ) {
    _ret(-1);
   }
 
-  if ( roar_vio_init_calls(tc) == -1 ) {
-   free(tc);
+  if ( roar_vio_clear_calls(tc) == -1 ) {
+   roar_mm_free(tc);
    _ret(-1);
   }
 
@@ -775,12 +808,12 @@ int     roar_vio_dstr_build_chain(struct roar_vio_dstr_chain * chain, struct roa
                    c->type & 0xFFFF, roar_vio_dstr_get_name(c->type), c->need_vio, c->def->o_flags);
 
   if ( c->need_vio ) {
-   if ( (tc = malloc(sizeof(struct roar_vio_calls))) == NULL ) {
+   if ( (tc = roar_mm_malloc(sizeof(struct roar_vio_calls))) == NULL ) {
     _ret(-1);
    }
 
-   if ( roar_vio_init_calls(tc) == -1 ) {
-    free(tc);
+   if ( roar_vio_clear_calls(tc) == -1 ) {
+    roar_mm_free(tc);
     _ret(-1);
    }
 
@@ -800,11 +833,20 @@ int     roar_vio_dstr_build_chain(struct roar_vio_dstr_chain * chain, struct roa
        _ret(-1);
       }
      break;
+#ifdef ROAR_HAVE_LIBZ
     case ROAR_VIO_DSTR_OBJT_GZIP:
-      if ( roar_vio_open_gzip(tc, prev, -1) == -1 ) {
+      //if ( roar_vio_open_gzip(tc, prev, -1) == -1 ) {
+      if ( roar_vio_open_zlib(tc, prev, -1, 1) == -1 ) {
        _ret(-1);
       }
      break;
+    case ROAR_VIO_DSTR_OBJT_ZLIB:
+      //if ( roar_vio_open_gzip(tc, prev, -1) == -1 ) {
+      if ( roar_vio_open_zlib(tc, prev, -1, 0) == -1 ) {
+       _ret(-1);
+      }
+     break;
+#endif
     case ROAR_VIO_DSTR_OBJT_BZIP2:
     case ROAR_VIO_DSTR_OBJT_PGP:
       if ( roar_vio_open_pgp_decrypt(tc, prev, NULL) == -1 ) {
@@ -845,6 +887,14 @@ int     roar_vio_dstr_build_chain(struct roar_vio_dstr_chain * chain, struct roa
        _ret(-1);
       }
      break;
+    case ROAR_VIO_DSTR_OBJT_NULL:
+    case ROAR_VIO_DSTR_OBJT_ZERO:
+    case ROAR_VIO_DSTR_OBJT_FULL:
+    case ROAR_VIO_DSTR_OBJT_NRANDOM:
+      if ( roar_vio_open_misc_by_name(tc, roar_vio_dstr_get_name(c->type)) == -1 ) {
+       _ret(-1);
+      }
+     break;
     case ROAR_VIO_DSTR_OBJT_SSL1:
     case ROAR_VIO_DSTR_OBJT_SSL2:
     case ROAR_VIO_DSTR_OBJT_SSL3:
