@@ -1,7 +1,7 @@
 //alaw.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of libroardsp a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -819,5 +819,61 @@ int roardsp_conv_pcm162alaw (char * out, int16_t * in, size_t len) {
 #endif
 }
 
+int roardsp_conv_autlaw2pcm32 (int32_t * out, int16_t * in, size_t len) {
+#ifdef ROAR_SUPPORT_ALAW
+ uint16_t inp;
+ register unsigned char alaw, extra;
+ register int bits;
+ uint32_t op;
+ int i;
+
+ for (i = len-1; i > -1; i--) {
+  inp = ROAR_NET2HOST16(in[i]);
+  alaw = (inp & 0xFF00) >> 8;
+  extra = inp & 0x00FF;
+
+//  printf("Balaw=%u\n", (unsigned)alaw);
+
+  bits = (alaw & 0x70) >> 4;
+  if ( bits == 0 )
+   bits = 1;
+
+  op = (uint32_t)_roardsp_alaw2pcm16[(int)alaw] << 16;
+//  out[i] |= extra << (11 + bits);
+  out[i]  = op;
+ }
+
+ return 0;
+#else
+ return -1;
+#endif
+}
+
+int roardsp_conv_pcm322autlaw (int16_t * out, int32_t * in, size_t len) {
+#ifdef ROAR_SUPPORT_ALAW_RW
+ register unsigned int i;
+ register uint16_t upper, r;
+ register unsigned char alaw;
+ register int bits;
+
+ for (i = 0; i < len; i++) {
+  upper = (uint32_t)((in[i] & 0xFFFF0000L) >> 16);
+  alaw = _roardsp_pcm132alaw[(upper + 32768) >> 3];
+//  printf("Aalaw=%u\n", (unsigned)alaw);
+  bits = (alaw & 0x70) >> 4;
+  if ( bits == 0 )
+   bits = 1;
+
+  r  = ((in[i] << (6 + bits)) & 0xFF000000) >> 24;
+  r |= alaw << 8;
+
+  out[i] = ROAR_HOST2NET16(r);
+ }
+
+ return 0;
+#else
+ return -1;
+#endif
+}
 
 //ll
