@@ -1,7 +1,7 @@
 //driver_roar.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2011
  *
  *  This file is part of roarclients a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -26,7 +26,7 @@
 #include <roaraudio.h>
 #include "driver.h"
 
-int roar_cdriver_roar(struct roar_vio_calls * calls, char * name, char * dev, struct roar_audio_info * info, int dir) {
+int roar_cdriver_roar(struct roar_vio_calls * calls, const char * name, char * dev, struct roar_audio_info * info, int dir) {
  return roar_vio_simple_stream(calls, info->rate, info->channels, info->bits, info->codec, dev, dir, "libroareio client");
 }
 
