@@ -1,7 +1,7 @@
 //libroaross.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2010-2011
  *
  *  This file is part of libroar a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -179,9 +179,11 @@ static struct {
  int     (*access)(const char *pathname, int mode);
  int     (*open64)(const char *__file, int __oflag, ...);
  int     (*creat)(const char *_CREAT_ARG_PATHNAME, mode_t mode);
+#ifndef __clang__
  int     (*stat)(const char *path, struct stat *buf);
  int     (*fstat)(int filedes, struct stat *buf);
  int     (*lstat)(const char *path, struct stat *buf);
+#endif
 } _os;
 
 static struct {
@@ -275,6 +277,9 @@ static struct devices {
  {"/dev/sndstat",       HT_STATIC,    sizeof(_sf__dev_sndstat)-1, _sf__dev_sndstat, NULL},
 #ifdef ROAR_DEFAULT_OSS_DEV
  {ROAR_DEFAULT_OSS_DEV, HT_WAVEFORM,  0, NULL, NULL},
+#endif
+#ifdef ROAR_DEFAULT_OSS_MIX_DEV
+ {ROAR_DEFAULT_OSS_MIX_DEV, HT_MIXER, 0, NULL, NULL},
 #endif
  {NULL, HT_NONE, 0, NULL, NULL},
 };
@@ -304,9 +309,11 @@ static void _init_os (void) {
  _os.access = dlsym(REAL_LIBC, "access");
  _os.open64 = dlsym(REAL_LIBC, "open64");
  _os.creat  = dlsym(REAL_LIBC, "creat");
+#ifndef __clang__
  _os.stat   = dlsym(REAL_LIBC, "stat");
  _os.fstat  = dlsym(REAL_LIBC, "fstat");
  _os.lstat  = dlsym(REAL_LIBC, "lstat");
+#endif
 }
 
 static void _init_ptr (void) {
@@ -629,8 +636,18 @@ static int _open_file (const char *pathname, int flags) {
  }
 #endif
 
- if ( (flags & O_DIRECTORY) || (flags & O_EXCL) ) {
-  ROAR_DBG("_open_file(pathname='%s', flags=0x%x) = -1 // invalid flags (O_DIRECTORY or O_EXCL)", pathname, flags);
+ if ( flags & O_DIRECTORY ) {
+  ROAR_DBG("_open_file(pathname='%s', flags=0x%x) = -1 // invalid flags (O_DIRECTORY)", pathname, flags);
+  errno = EINVAL;
+  return -1;
+ }
+
+ if ( flags & O_EXCL ) {
+  ROAR_WARN("_open_file(pathname='%s', flags=0x%x): This application is asked us for exclusive device access.", pathname, flags);
+  ROAR_WARN("_open_file(pathname='%s', flags=0x%x): This is maybe not what you want.", pathname, flags);
+  ROAR_WARN("_open_file(pathname='%s', flags=0x%x): We reject this according to OSS specs.", pathname, flags);
+  ROAR_WARN("_open_file(pathname='%s', flags=0x%x): There should be a option in the application to switch this off.", pathname, flags);
+  ROAR_WARN("_open_file(pathname='%s', flags=0x%x) = -1 // invalid flags (O_EXCL)", pathname, flags);
   errno = EINVAL;
   return -1;
  }
@@ -775,7 +792,7 @@ int libroaross_open_vio(struct handle ** handleret, struct roar_vio_calls ** vio
  handle->type        = HT_VIO;
  handle->sysio_flags = flags;
 
- if ( roar_vio_init_calls(&(handle->stream_vio)) == -1 ) {
+ if ( roar_vio_clear_calls(&(handle->stream_vio)) == -1 ) {
   _close_handle(handle);
   return -1;
  }
@@ -1480,6 +1497,10 @@ IOCTL() {
       case SNDCTL_DSP_SETTRIGGER: // we should implement this using PAUSE flag.
         return 0;
        break;
+      case SNDCTL_DSP_SETDUPLEX:
+        ROAR_WARN("ioctl(__fd=%i, __request=0x%lX (SNDCTL_DSP_SETDUPLEX)): This call is obsolete since end of 90's. Stop using it.", __fd, (long unsigned int) __request);
+        return 0;
+       break;
       case SNDCTL_DSP_SPEED:
         handle->stream.info.rate = *ip;
         ROAR_DBG("ioctl(__fd=%i, __request=0x%lX): rate=%i", __fd, (long unsigned int) __request, *ip);
@@ -1742,6 +1763,19 @@ int select(int nfds, fd_set *readfds, fd_set *writefds,
  if ( ret < 1 ) {
   roar_mm_free(sv);
   ROAR_DBG("select(nfds=%i, readfds=%p, writefds=%p, exceptfds=%p, timeout=%p) = %i", nfds, readfds, writefds, exceptfds, timeout, (int)ret);
+
+  if ( ret == 0 ) {
+   if ( readfds != NULL ) {
+    FD_ZERO(readfds);
+   }
+   if ( writefds != NULL ) {
+    FD_ZERO(writefds);
+   }
+   if ( exceptfds != NULL ) {
+    FD_ZERO(exceptfds);
+   }
+  }
+
   return ret;
  }
 
@@ -2005,6 +2039,7 @@ int creat(const char *_CREAT_ARG_PATHNAME, mode_t mode) {
 // emulated *stat*() functions follow:
 // -------------------------------------
 
+#ifndef __clang__
 int stat(const char *path, struct stat *buf) {
  struct devices * ptr;
 
@@ -2040,6 +2075,7 @@ int lstat(const char *path, struct stat *buf) {
 
  return _os.lstat(path, buf);
 }
+#endif
 
 // -------------------------------------
 // emulated stdio functions follow:
