@@ -1,7 +1,7 @@
 //roarctl.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of roarclients a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -23,6 +23,10 @@
  *
  */
 
+/* ckport options:
+ * ckport: ignore-symbol: roar_req of target libroar0 -- Used within ping command.
+ */
+
 #define _UNITS_T_BASE_USEC
 
 #include <roaraudio.h>
@@ -47,6 +51,8 @@
 
 int g_verbose = 0;
 
+void display_client (struct roar_connection * con, int id);
+void display_stream (struct roar_connection * con, int id);
 int display_mixer (struct roar_connection * con, int stream);
 int show_meta_all (struct roar_connection * con, int id);
 
@@ -59,6 +65,9 @@ void usage (void) {
         "  --help                  - Show this help\n"
         "  --verbose   -v          - Show verbose output\n"
         "  --list-aiprofiles       - Show audio info profiles and exit\n"
+        "  --list-libstandards     - Gets list of library (libroar) supported standards\n"
+        "  --enum-servers          - Show a list of possible servers\n"
+        "  --hash-password         - Hash a password and exit\n"
        );
 
  printf("\nCommands:\n\n");
@@ -95,6 +104,11 @@ void usage (void) {
         "  flag   ID FLAGS         - Sets flags FLAGS on stream ID. FLAGS may be a comma\n"
         "                            separated list of flags.\n"
         "  unflag ID FLAGS         - Unsets flags on a stream. See flag.\n"
+        "  toogleflag ID FLAGS     - Toggels flags on a stream. See flag.\n"
+        "  protectflag ID FLAGS    - Protects flags on a stream. Those flags can not be\n"
+        "                            changed anymore after the they got protected. See flag.\n"
+        "\n"
+        "  role ID ROLE            - Set role ROLE on stream ID.\n"
         "\n"
         "  kick TYPE ID            - Kicks object of TYPE with id ID\n"
         "                            Types: client stream sample source\n"
@@ -109,13 +123,94 @@ void usage (void) {
         "  metasave ID FILE        - Saves meta data of stream ID to file FILE\n"
         "  metaload ID FILE        - Loads meta data from file FILE and set it on stream ID\n"
         "\n"
+        "  serverinfo              - Gets general information about the server\n"
+        "  servertime              - Gets server's time\n"
         "  serveroinfo             - Gets Information about server output\n"
+        "  serveroinfo2 DIR        - Gets Information about server output for stream direction dir\n"
+        "\n"
+        "  serverstandards         - Gets list of server supported standards\n"
+        "  libstandards            - Gets list of library (libroar) supported standards\n"
+        "\n"
         "  listclients             - Gets Information about clients\n"
+        "  clientinfo ID           - Gets Information about client ID\n"
         "  liststreams             - Gets Information about streams\n"
+        "  streaminfo ID           - Gets Information about stream ID\n"
         "  allinfo                 - Get all infos\n"
        );
 }
 
+static void _bin2hex(char * out, char * in, size_t inlen, int uppercase) {
+ const char * tab = uppercase ? "0123456789ABCDEF" : "0123456789abcdef";
+ unsigned char c;
+ int nib0, nib1;
+
+ for (; inlen; inlen--) {
+  c = *(in++);
+  nib0 = (c & 0x0F) >> 0;
+  nib1 = (c & 0xF0) >> 4;
+//  printf("inlen=%zu, c=%u, nibs={%i, %i}\n", inlen, (unsigned)c, nib1, nib0);
+  *(out++) = tab[nib1];
+  *(out++) = tab[nib0];
+ }
+ *out = 0;
+}
+
+int hash_password (void) {
+ char * pw = NULL;
+ char salt[8];
+ char digest[24];
+ char out[2*(8+24)+3+5+1] = "tiger:";
+
+ if ( roar_passwd_simple_ask_pw(&pw, "Password to hash?", NULL) == -1 )
+  return -1;
+
+ roar_random_gen_nonce(salt, sizeof(salt));
+
+ roar_hash_salted_buffer(digest, pw, roar_mm_strlen(pw), ROAR_HT_TIGER, salt, sizeof(salt));
+
+ roar_mm_free(pw);
+
+ _bin2hex(out+6, salt, sizeof(salt), 0);
+ out[6+16] = ':';
+ _bin2hex(out+6+16+1, digest, sizeof(digest), 0);
+
+ out[sizeof(out)-1] = 0;
+
+ printf("Password: %s\n", out);
+
+ return 0;
+}
+
+int enum_servers (void) {
+ struct roar_server * list;
+ struct roar_server * c;
+ int flags    = ROAR_ENUM_FLAG_DESC|ROAR_ENUM_FLAG_LOCATION;
+ int dir      = -1;
+ int socktype = -1;
+ int i;
+
+ if ( (list = roar_enum_servers(flags, dir, socktype)) == NULL )
+  return -1;
+
+ printf("Server           Location         Description\n");
+ printf("----------------------------------------------------------------------\n");
+
+ for (i = 0; ; i++) {
+  c = &(list[i]);
+  printf("%-16s %-16s %s\n",
+             c->server      == NULL ? "(default)" : c->server,
+             c->location    == NULL ? ""          : c->location,
+             c->description == NULL ? ""          : c->description
+        );
+  if ( c->server == NULL )
+   break;
+ }
+
+ roar_enum_servers_free(list);
+
+ return 0;
+}
+
 #ifdef ROAR_HAVE_GETTIMEOFDAY
 int ping (struct roar_connection * con, int num) {
  struct timeval         try, ans;
@@ -131,7 +226,8 @@ int ping (struct roar_connection * con, int num) {
   memset(&m, 0, sizeof(m));
 
   m.cmd = ROAR_CMD_NOOP;
-  m.datalen = 0;
+  m.stream  = -1;
+  m.datalen =  0;
 
   // we use roar_req() directly here because of speed.
   // roar_noop() does basicly the same but is a bit slower.
@@ -158,7 +254,7 @@ int ping (struct roar_connection * con, int num) {
    max = cur;
 
   if ( i != (num - 1) )
-   sleep(1);
+   roar_sleep(1);
  }
 
  printf("\n--- ping statistics ---\n");
@@ -169,16 +265,77 @@ int ping (struct roar_connection * con, int num) {
 }
 #endif
 
-void server_oinfo (struct roar_connection * con) {
+#define _pm(m,n) if ( info->m != NULL ) printf("Server %-15s: %s\n", (n), (info->m));
+void server_info (struct roar_connection * con) {
+ struct roar_server_info * info = roar_server_info(con);
+
+ if ( info == NULL ) {
+  fprintf(stderr, "Error: can not get server info\n");
+  return;
+ }
+
+ _pm(version, "version");
+ _pm(location, "location");
+ _pm(description, "description");
+ _pm(contact, "contact");
+ _pm(serial, "serial");
+ _pm(address, "address");
+ _pm(uiurl, "UI URL");
+ _pm(un.sysname, "System sysname");
+ _pm(un.release, "System release");
+ _pm(un.nodename, "System nodename");
+ _pm(un.machine, "System machine");
+
+ roar_server_info_free(info);
+}
+#undef _pm
+
+void server_time (struct roar_connection * con) {
+ struct roar_time time;
+
+ if ( roar_get_time(con, &time) == -1 ) {
+  fprintf(stderr, "Error: can not get server time: %s\n", roar_vs_strerr(roar_error));
+  return;
+ }
+
+ if ( g_verbose ) {
+  printf("Server time           : %llu.%06llu [+%llu/2^64] sec\n",
+         (long long unsigned int)time.t_sec,
+         (long long unsigned int) time.t_ssec / 18446744073709LLU,
+         (long long unsigned int) time.t_ssec);
+ } else {
+  if ( time.c_freq == 1000000000LL && time.t_ssec == 0 ) {
+   printf("Server time           : %llu sec\n",
+          (long long unsigned int)time.t_sec);
+  } else {
+   printf("Server time           : %llu.%06llu sec\n",
+          (long long unsigned int)time.t_sec,
+          (long long unsigned int) time.t_ssec / 18446744073709LLU);
+  }
+ }
+ if ( time.c_freq )
+  printf("Server clock frequency: %f Hz\n", (float)((double)time.c_freq / (double)1000000000.));
+ if ( time.c_drift ) {
+#ifdef ROAR_HAVE_LIBM
+  printf("Server clock drift    : %llu:2^64 (~10^%f)\n",
+         (long long unsigned int)time.c_drift, logf((float)time.c_drift)/2.302585f);
+#else
+  printf("Server clock drift    : %llu:2^64\n",
+         (long long unsigned int)time.c_drift);
+#endif
+ }
+}
+
+void server_oinfo (struct roar_connection * con, int dir) {
  struct roar_stream s;
 
- if ( roar_server_oinfo(con, &s) == -1 ) {
+ if ( roar_server_oinfo2(con, &s, dir) == -1 ) {
   fprintf(stderr, "Error: can not get server output info\n");
   return;
  }
 
  printf("Stream direction      : %s\n", roar_dir2str(s.dir));
- printf("Server Output rate    : %i\n", s.info.rate);
+ printf("Server Output rate    : %i Hz\n", s.info.rate);
  printf("Server Output bits    : %i\n", s.info.bits);
  printf("Server Output channels: %i\n", s.info.channels);
  printf("Server Output codec   : %i (%s%s)\n", s.info.codec, roar_codec2str(s.info.codec),
@@ -188,6 +345,51 @@ void server_oinfo (struct roar_connection * con) {
    printf("Server Position       : %lu S (%.3fs)\n", (unsigned long int) s.pos, (float)s.pos/(s.info.rate*s.info.channels));
 }
 
+static void print_stds (const struct roar_stds * stds) {
+ size_t i;
+ int vendor, standard, version;
+ char numbuf[2][8];
+ const char * vendor_name;
+
+ for (i = 0; i < stds->stds_len; i++) {
+  vendor   = ROAR_STD_VENDOR(stds->stds[i]);
+  standard = ROAR_STD_STD(stds->stds[i]);
+  version  = ROAR_STD_VERSION(stds->stds[i]);
+
+  if ( (vendor_name = roar_stds_vendor2str(vendor)) == NULL ) {
+   snprintf(numbuf[0], sizeof(numbuf[0]), "%i", vendor);
+   numbuf[0][sizeof(numbuf[0])-1] = 0;
+   vendor_name = numbuf[0];
+  }
+
+  if ( version == 0 ) {
+   numbuf[1][0] = 0;
+  } else {
+   snprintf(numbuf[1], sizeof(numbuf[1]), "-%i", version);
+   numbuf[1][sizeof(numbuf[1])-1] = 0;
+  }
+
+  printf("Server standard       : %s-%i%s\n", vendor_name, standard, numbuf[1]);
+ }
+}
+
+void server_standards (struct roar_connection * con) {
+ struct roar_stds * stds;
+
+ if ( roar_caps_stds(con, &stds, NULL, -1) == -1 ) {
+  fprintf(stderr, "Error: can not get server standards\n");
+  return;
+ }
+
+ print_stds(stds);
+
+ roar_stds_free(stds);
+}
+
+void lib_standards (void) {
+ print_stds(libroar_libstds());
+}
+
 const char * proc_name (pid_t pid) {
  static char ret[80] = "?";
 #ifdef __linux__
@@ -220,21 +422,7 @@ const char * proc_name (pid_t pid) {
 void list_clients (struct roar_connection * con) {
  int i;
  int num;
- int h;
  int id[ROAR_CLIENTS_MAX];
- char tmp[80];
- int self_id;
- struct roar_client self_client;
- struct roar_client c;
-#ifdef _POSIX_USERS
- struct group  * grp = NULL;
- struct passwd * pwd = NULL;
-#endif
-
- if ( (self_id = roar_get_clientid(con)) != -1 ) {
-  if ( roar_get_client(con, &self_client, self_id) == -1 )
-   self_id = -1;
- }
 
  if ( (num = roar_list_clients(con, id, ROAR_CLIENTS_MAX)) == -1 ) {
   fprintf(stderr, "Error: can not get client list\n");
@@ -242,90 +430,104 @@ void list_clients (struct roar_connection * con) {
  }
 
  for (i = 0; i < num; i++) {
-  printf("client %i:\n", id[i]);
-  if ( roar_get_client(con, &c, id[i]) == -1 ) {
-   fprintf(stderr, "Error: can not get client info\n");
-   continue;
-  }
+  display_client(con, id[i]);
+ }
+}
 
-  if ( c.name[0] != '\0' )
-   printf("Client name           : %s\n", c.name);
-
-  if ( roar_nnode_get_socktype(&(c.nnode)) != ROAR_SOCKET_TYPE_UNKNOWN ) {
-   if ( roar_nnode_to_str(&(c.nnode), tmp, 80) == 0 ) {
-    printf("Client network node   : %s\n", tmp);
-   }
-  }
-
-  if ( c.pid != -1 ) {
-   if ( self_id != -1 && roar_nnode_cmp(&(self_client.nnode), &(c.nnode)) == 0 ) {
-    printf("Client PID            : %i(%s)\n", c.pid, proc_name(c.pid));
-   } else { 
-    printf("Client PID            : %i\n", c.pid);
-   }
-  }
-  if ( c.uid != -1 ) {
+void display_client (struct roar_connection * con, int id) {
+ static int self_id = -1;
+ static struct roar_client   self_client_store;
+ static struct roar_client * self_client = NULL;
+ struct roar_client c;
+ char tmp[80];
+ int h;
 #ifdef _POSIX_USERS
-   if ( self_id != -1 && roar_nnode_cmp(&(self_client.nnode), &(c.nnode)) == 0 ) {
-    pwd = getpwuid(c.uid);
-    grp = getgrgid(c.gid);
-    printf("Client UID/GID        : %i(%s)/%i(%s)\n", c.uid, pwd ? pwd->pw_name : "?", c.gid, grp ? grp->gr_name : "?");
-   } else {
-#else
-   if ( 1 ) {
+ struct group  * grp = NULL;
+ struct passwd * pwd = NULL;
 #endif
-    printf("Client UID/GID        : %i/%i\n", c.uid, c.gid);
-   }
-  }
 
-  if ( g_verbose && c.proto != ROAR_PROTO_NONE ) {
-   printf("Client Protocol       : %s\n", roar_proto2str(c.proto));
-  }
-
-  if ( g_verbose && c.byteorder != ROAR_BYTEORDER_UNKNOWN ) {
-   if ( c.byteorder == ROAR_BYTEORDER_NETWORK ) {
-    strcpy(tmp, " (network byteorder");
+ if ( self_id == -1 ) {
+  if ( (self_id = roar_get_clientid(con)) != -1 ) {
+   if ( roar_get_client(con, &self_client_store, self_id) == -1 ) {
+    self_id = -1;
    } else {
-    *tmp = 0;
+    self_client = &self_client_store;
    }
-
-   if ( c.byteorder == ROAR_BYTEORDER_NATIVE ) {
-    if ( *tmp ) {
-     strcat(tmp, ", native");
-    } else {
-     strcpy(tmp, " (native");
-    }
-   }
-
-   if ( *tmp )
-    strcat(tmp, ")");
-
-   printf("Client Byteorder      : %s%s\n", roar_byteorder2str(c.byteorder), tmp);
   }
-
-  if ( c.execed != -1 )
-   printf("Execed stream         : %i\n", c.execed);
-
-  for (h = 0; h < ROAR_CLIENTS_MAX_STREAMS_PER_CLIENT; h++)
-   if ( c.streams[h] != -1 )
-    printf("stream                : %i\n", c.streams[h]);
  }
 
+ printf("client %i:\n", id);
+ if ( roar_get_client(con, &c, id) == -1 ) {
+  fprintf(stderr, "Error: can not get client info\n");
+  return;
+ }
+
+ if ( c.name[0] != '\0' )
+  printf("Client name           : %s\n", c.name);
+  if ( roar_nnode_get_socktype(&(c.nnode)) != ROAR_SOCKET_TYPE_UNKNOWN ) {
+  if ( roar_nnode_to_str(&(c.nnode), tmp, sizeof(tmp)) == 0 ) {
+   printf("Client network node   : %s\n", tmp);
+  }
+ }
+
+ if ( c.pid != -1 ) {
+  if ( self_id != -1 && roar_nnode_cmp(&(self_client->nnode), &(c.nnode)) == 0 ) {
+   printf("Client PID            : %i(%s)\n", c.pid, proc_name(c.pid));
+  } else { 
+   printf("Client PID            : %i\n", c.pid);
+  }
+ }
+ if ( c.uid != -1 ) {
+#ifdef _POSIX_USERS
+  if ( self_id != -1 && roar_nnode_cmp(&(self_client->nnode), &(c.nnode)) == 0 ) {
+   pwd = getpwuid(c.uid);
+   grp = getgrgid(c.gid);
+   printf("Client UID/GID        : %i(%s)/%i(%s)\n", c.uid, pwd ? pwd->pw_name : "?", c.gid, grp ? grp->gr_name : "?");
+  } else {
+#else
+  if ( 1 ) {
+#endif
+   printf("Client UID/GID        : %i/%i\n", c.uid, c.gid);
+  }
+ }
+
+ if ( g_verbose && c.proto != ROAR_PROTO_NONE ) {
+  printf("Client Protocol       : %s\n", roar_proto2str(c.proto));
+ }
+
+ if ( g_verbose && c.byteorder != ROAR_BYTEORDER_UNKNOWN ) {
+  if ( c.byteorder == ROAR_BYTEORDER_NETWORK ) {
+   roar_mm_strscpy(tmp, " (network byteorder");
+  } else {
+   *tmp = 0;
+  }
+
+  if ( c.byteorder == ROAR_BYTEORDER_NATIVE ) {
+   if ( *tmp ) {
+    roar_mm_strscat(tmp, ", native");
+   } else {
+    roar_mm_strscpy(tmp, " (native");
+   }
+  }
+
+  if ( *tmp )
+   roar_mm_strscat(tmp, ")");
+
+  printf("Client Byteorder      : %s%s\n", roar_byteorder2str(c.byteorder), tmp);
+ }
+
+ if ( c.execed != -1 )
+  printf("Execed stream         : %i\n", c.execed);
+
+ for (h = 0; h < ROAR_CLIENTS_MAX_STREAMS_PER_CLIENT; h++)
+  if ( c.streams[h] != -1 )
+   printf("stream                : %i\n", c.streams[h]);
 }
 
 void list_streams (struct roar_connection * con) {
  int i;
  int num;
  int id[ROAR_STREAMS_MAX];
- char chanmap[ROAR_MAX_CHANNELS];
- struct roar_stream s;
- struct roar_stream_info info;
- char buffer[1024];
- char * flags = buffer;
- char * name  = buffer;
- char * infotext;
- size_t len;
-
 
  if ( (num = roar_list_streams(con, id, ROAR_STREAMS_MAX)) == -1 ) {
   fprintf(stderr, "Error: can not get stream list\n");
@@ -333,137 +535,162 @@ void list_streams (struct roar_connection * con) {
  }
 
  for (i = 0; i < num; i++) {
-  printf("stream %i:\n", id[i]);
-  if ( roar_get_stream(con, &s, id[i]) == -1 ) {
-   fprintf(stderr, "Error: can not get stream info\n");
-   continue;
-  }
-  printf("Stream direction      : %s\n", roar_dir2str(s.dir));
+  display_stream(con, id[i]);
+ }
+}
 
-  if ( roar_stream_get_name(con, &s, name, 1024) == 0 )
-   printf("Stream name           : %s\n", name);
+void display_stream (struct roar_connection * con, int id) {
+ char chanmap[ROAR_MAX_CHANNELS];
+ struct roar_stream s;
+ struct roar_stream_info info;
+ char buffer[1024];
+ char * flags = buffer;
+ char * name  = buffer;
+ char * infotext;
+ const char * mime;
+ size_t len;
 
-  if ( (int)s.pos_rel_id == -1 ) {
-   printf("Relativ position id   : none (stream not synchronized)\n");
-  } else if ( (int)s.pos_rel_id == id[i] ) {
-   printf("Relativ position id   : %i (self synchronized)\n", s.pos_rel_id);
+ printf("stream %i:\n", id);
+ if ( roar_get_stream(con, &s, id) == -1 ) {
+  fprintf(stderr, "Error: can not get stream info\n");
+  return;
+ }
+ printf("Stream direction      : %s\n", roar_dir2str(s.dir));
+
+ if ( roar_stream_get_name(con, &s, name, sizeof(buffer)) == 0 )
+  printf("Stream name           : %s\n", name);
+
+ if ( (int)s.pos_rel_id == -1 ) {
+  printf("Relativ position id   : none (stream not synchronized)\n");
+ } else if ( (int)s.pos_rel_id == id ) {
+  printf("Relativ position id   : %i (self synchronized)\n", s.pos_rel_id);
+ } else {
+  printf("Relativ position id   : %i (synchronized)\n", s.pos_rel_id);
+ }
+ if ( g_verbose > 1 && s.pos != (uint32_t)-1 ) {
+  if ( s.info.rate && s.info.channels ) {
+   printf("Position              : %lu S (%.3fs)\n", (unsigned long int) s.pos,
+                                   (float)s.pos/(s.info.rate*s.info.channels));
   } else {
-   printf("Relativ position id   : %i (synchronized)\n", s.pos_rel_id);
-  }
-  if ( g_verbose > 1 && s.pos != (uint32_t)-1 ) {
-   if ( s.info.rate && s.info.channels ) {
-    printf("Position              : %lu S (%.3fs)\n", (unsigned long int) s.pos,
-                                    (float)s.pos/(s.info.rate*s.info.channels));
-   } else {
-    printf("Position              : %lu S\n", (unsigned long int) s.pos);
-   }
-  }
-
-  switch (s.dir) {
-   case ROAR_DIR_MIDI_IN:
-   case ROAR_DIR_MIDI_OUT:
-     infotext = " ticks/s";
-    break;
-   case ROAR_DIR_LIGHT_IN:
-   case ROAR_DIR_LIGHT_OUT:
-     infotext = " updates/s";
-    break;
-   default:
-     infotext = " Hz";
-  }
-  if ( s.info.rate )
-   printf("Stream sample rate    : %i%s\n", s.info.rate, infotext);
-  if ( s.info.bits )
-   printf("Stream bits           : %i\n", s.info.bits);
-  if ( s.info.channels )
-  printf("Stream channels       : %i\n", s.info.channels);
-
-  printf("Stream codec          : %2i (%s%s)\n", s.info.codec, roar_codec2str(s.info.codec),
-                                       s.info.codec == ROAR_CODEC_DEFAULT ? " native" : "");
-  if ( roar_stream_get_info(con, &s, &info) != -1 ) {
-   if ( info.codec != s.info.codec ) {
-    printf("Streamed codec        : %2i (%s%s)\n", info.codec, roar_codec2str(info.codec),
-                                       info.codec == ROAR_CODEC_DEFAULT ? " native" : "");
-   }
-
-   if ( g_verbose ) {
-    if ( info.block_size )
-     printf("Stream block size     : %i Byte\n", info.block_size);
-
-    printf("Underruns pre/post    : %i/%i\n",   info.pre_underruns, info.post_underruns);
-    if ( g_verbose > 1 )
-     printf("Stream delay          : %ims (%.2fm)\n",   (int)info.delay/1000, (info.delay*(float)_SPEED_OF_SOUND));
-
-    if ( g_verbose > 1 )
-     printf("Stream mixer          : %i\n",   info.mixer);
-
-    if ( g_verbose > 1 )
-     printf("Stream state          : %s\n",   roar_streamstate2str(info.state));
-
-    if ( g_verbose > 1 )
-     printf("Stream role           : %s\n",   roar_role2str(info.role));
-
-    *flags = 0;
-    if ( info.flags & ROAR_FLAG_PRIMARY )
-     strcat(flags, "primary ");
-    if ( info.flags & ROAR_FLAG_SYNC )
-     strcat(flags, "sync ");
-    if ( info.flags & ROAR_FLAG_OUTPUT )
-     strcat(flags, "output ");
-    if ( info.flags & ROAR_FLAG_SOURCE )
-     strcat(flags, "source ");
-    if ( info.flags & ROAR_FLAG_META )
-     strcat(flags, "meta ");
-    if ( info.flags & ROAR_FLAG_AUTOCONF )
-     strcat(flags, "autoconf ");
-    if ( info.flags & ROAR_FLAG_CLEANMETA )
-     strcat(flags, "cleanmeta ");
-    if ( info.flags & ROAR_FLAG_HWMIXER )
-     strcat(flags, "hwmixer ");
-    if ( info.flags & ROAR_FLAG_PAUSE )
-     strcat(flags, "pause ");
-    if ( info.flags & ROAR_FLAG_MUTE )
-     strcat(flags, "mute ");
-    if ( info.flags & ROAR_FLAG_MMAP )
-     strcat(flags, "mmap ");
-    if ( info.flags & ROAR_FLAG_ANTIECHO )
-     strcat(flags, "antiecho ");
-    if ( info.flags & ROAR_FLAG_VIRTUAL )
-     strcat(flags, "virtual ");
-    if ( info.flags & ROAR_FLAG_RECSOURCE )
-     strcat(flags, "recsource ");
-    if ( info.flags & ROAR_FLAG_PASSMIXER )
-     strcat(flags, "passmixer ");
-    if ( info.flags & ROAR_FLAG_PRETHRU )
-     strcat(flags, "prethru ");
-    if ( info.flags & ROAR_FLAG_IMMUTABLE )
-     strcat(flags, "immutable ");
-    if ( info.flags & ROAR_FLAG_ENHANCE )
-     strcat(flags, "enhance ");
-
-    printf("Flags                 : %s\n", flags);
-   }
-  }
-
-  if ( g_verbose ) {
-   len = ROAR_MAX_CHANNELS;
-   if ( roar_stream_get_chanmap(con, &s, chanmap, &len) == -1 ) {
-    fprintf(stderr, "Error: can not get stream channel map\n");
-   } else {
-    if ( roardsp_chanlist2str(chanmap, len, buffer, 1024) == -1 ) {
-     fprintf(stderr, "Error: can not convert channel map into string\n");
-    } else {
-     printf("Channel Map           : %s\n", buffer);
-    }
-   }
-  }
-
-  if ( s.dir != ROAR_DIR_THRU ) {
-   display_mixer(con, id[i]);
-   show_meta_all(con, id[i]);
+   printf("Position              : %lu S\n", (unsigned long int) s.pos);
   }
  }
 
+ switch (s.dir) {
+  case ROAR_DIR_MIDI_IN:
+  case ROAR_DIR_MIDI_OUT:
+    infotext = " ticks/s";
+   break;
+  case ROAR_DIR_LIGHT_IN:
+  case ROAR_DIR_LIGHT_OUT:
+    infotext = " updates/s";
+   break;
+  default:
+    infotext = " Hz";
+ }
+ if ( s.info.rate )
+  printf("Stream sample rate    : %i%s\n", s.info.rate, infotext);
+ if ( s.info.bits )
+  printf("Stream bits           : %i\n", s.info.bits);
+ if ( s.info.channels )
+  printf("Stream channels       : %i\n", s.info.channels);
+
+ mime = roar_codec2mime(s.info.codec);
+ printf("Stream codec          : %2i (%s%s%s%s)\n", s.info.codec, roar_codec2str(s.info.codec),
+                                      s.info.codec == ROAR_CODEC_DEFAULT ? " native" : "",
+                                      mime == NULL ? "" : " mimetype:",
+                                      mime == NULL ? "" : mime
+                                     );
+ if ( roar_stream_get_info(con, &s, &info) != -1 ) {
+  if ( info.codec != s.info.codec ) {
+   mime = roar_codec2mime(info.codec);
+   printf("Streamed codec        : %2i (%s%s%s%s)\n", info.codec, roar_codec2str(info.codec),
+                                      info.codec == ROAR_CODEC_DEFAULT ? " native" : "",
+                                      mime == NULL ? "" : " mimetype:",
+                                      mime == NULL ? "" : mime
+                                     );
+  }
+
+  if ( g_verbose ) {
+   if ( info.block_size )
+    printf("Stream block size     : %i Byte\n", info.block_size);
+
+   printf("Underruns pre/post    : %i/%i\n",   info.pre_underruns, info.post_underruns);
+   if ( g_verbose > 1 )
+    printf("Stream delay          : %ims (%.2fm)\n",   (int)info.delay/1000, (info.delay*(float)_SPEED_OF_SOUND));
+
+   if ( g_verbose > 1 )
+    printf("Stream mixer          : %i\n",   info.mixer);
+
+   if ( g_verbose > 1 )
+    printf("Stream state          : %s\n",   roar_streamstate2str(info.state));
+
+   if ( g_verbose > 1 )
+    printf("Stream role           : %s\n",   roar_role2str(info.role));
+
+   *flags = 0;
+#define _strcat(dst,src) roar_mm_strlcat((dst),(src), sizeof(buffer))
+   if ( info.flags & ROAR_FLAG_PRIMARY )
+    _strcat(flags, "primary ");
+   if ( info.flags & ROAR_FLAG_SYNC )
+    _strcat(flags, "sync ");
+   if ( info.flags & ROAR_FLAG_OUTPUT )
+    _strcat(flags, "output ");
+   if ( info.flags & ROAR_FLAG_SOURCE )
+    _strcat(flags, "source ");
+   if ( info.flags & ROAR_FLAG_META )
+    _strcat(flags, "meta ");
+   if ( info.flags & ROAR_FLAG_AUTOCONF )
+    _strcat(flags, "autoconf ");
+   if ( info.flags & ROAR_FLAG_CLEANMETA )
+    _strcat(flags, "cleanmeta ");
+   if ( info.flags & ROAR_FLAG_HWMIXER )
+    _strcat(flags, "hwmixer ");
+   if ( info.flags & ROAR_FLAG_PAUSE )
+    _strcat(flags, "pause ");
+   if ( info.flags & ROAR_FLAG_MUTE )
+    _strcat(flags, "mute ");
+   if ( info.flags & ROAR_FLAG_MMAP )
+    _strcat(flags, "mmap ");
+   if ( info.flags & ROAR_FLAG_ANTIECHO )
+    _strcat(flags, "antiecho ");
+   if ( info.flags & ROAR_FLAG_VIRTUAL )
+    _strcat(flags, "virtual ");
+   if ( info.flags & ROAR_FLAG_RECSOURCE )
+    _strcat(flags, "recsource ");
+   if ( info.flags & ROAR_FLAG_PASSMIXER )
+    _strcat(flags, "passmixer ");
+   if ( info.flags & ROAR_FLAG_PRETHRU )
+    _strcat(flags, "prethru ");
+   if ( info.flags & ROAR_FLAG_IMMUTABLE )
+    _strcat(flags, "immutable ");
+   if ( info.flags & ROAR_FLAG_ENHANCE )
+    _strcat(flags, "enhance ");
+   if ( info.flags & ROAR_FLAG_SINGLESINK )
+    _strcat(flags, "singlesink ");
+#undef _strcat
+
+   printf("Flags                 : %s\n", flags);
+  }
+ }
+
+ if ( g_verbose ) {
+  len = ROAR_MAX_CHANNELS;
+  if ( roar_stream_get_chanmap(con, &s, chanmap, &len) == -1 ) {
+   fprintf(stderr, "Error: can not get stream channel map\n");
+  } else {
+   if ( roardsp_chanlist2str(chanmap, len, buffer, 1024) == -1 ) {
+    fprintf(stderr, "Error: can not convert channel map into string\n");
+   } else {
+    printf("Channel Map           : %s\n", buffer);
+   }
+  }
+ }
+
+ if ( s.dir != ROAR_DIR_THRU ) {
+  display_mixer(con, id);
+  show_meta_all(con, id);
+ }
 }
 
 int display_mixer (struct roar_connection * con, int stream) {
@@ -536,10 +763,10 @@ int set_mixer (struct roar_connection * con, int * cur, int max, char * arg[]) {
  int i;
  int len;
  int old_chans;
- int vol_l, vol_r, vol_mono;
  char * k;
  struct roar_mixer_settings mixer;
  struct roar_mixer_settings old_mixer;
+ int mode = ROAR_SET_VOL_ALL;
 
  if (*cur + 2 > max)
   return -1;
@@ -548,110 +775,60 @@ int set_mixer (struct roar_connection * con, int * cur, int max, char * arg[]) {
 
  k = arg[++(*cur)];
 
- if ( roar_get_vol(con, id, &old_mixer, &old_chans) == -1 ) {
-  fprintf(stderr, "Error: can not get stream mixer info for stream %i\n", id);
-  return -1;
- }
-
  if ( !strcmp(arg[*cur + 1], "scale") ) {
   (*cur)++; // 'scale'
   (*cur)++;
   scale = set_mixer_parse_volume(arg[*cur], strlen(arg[*cur]), 65535);
  }
 
-// TODO: clean up code here as the % vs. abs code is very duplicate...
-
- if ( strcmp(k, "mono") == 0 && old_chans != 1 ) {
-  chans = 1;
-
-  if ( *cur + 1 > max )
-   return -1;
-
-  k   = arg[++(*cur)];
-  len = strlen(k);
-
-  vol_mono = set_mixer_parse_volume(k, len, scale);
-
-  for (i = 0; i < old_chans; i++)
-   mixer.mixer[i] = vol_mono;
-
-  chans = old_chans;
-
- } else if ( strcmp(k, "stereo") == 0 && old_chans != 2 ) {
-  chans = old_chans;
-
-  if ( *cur + 2 > max )
-   return -1;
-
-  k   = arg[++(*cur)];
-  len = strlen(k);
-
-  vol_l = set_mixer_parse_volume(k, len, scale);
-
-  k   = arg[++(*cur)];
-  len = strlen(k);
-
-  vol_r = set_mixer_parse_volume(k, len, scale);
-
-  vol_mono = (vol_l + vol_r) / 2;
-
-  mixer.mixer[0] = vol_l;
-  mixer.mixer[1] = vol_r;
-
-  switch (chans) {
-   case 1:
-     mixer.mixer[0] = vol_mono;
-    break;
-//   case 2: classic stereo...
-   case 3:
-     mixer.mixer[2] = vol_mono;
-    break;
-   case 4:
-     mixer.mixer[2] = vol_l;
-     mixer.mixer[3] = vol_r;
-    break;
-   case 5:
-     mixer.mixer[2] = vol_mono;
-     mixer.mixer[3] = vol_l;
-     mixer.mixer[4] = vol_r;
-    break;
-   case 6:
-     mixer.mixer[2] = vol_mono;
-     mixer.mixer[3] = vol_mono;
-     mixer.mixer[4] = vol_l;
-     mixer.mixer[5] = vol_r;
-    break;
-   default:
-     ROAR_ERR("mode stereo not supported on stream with %i channels", chans);
-     return -1;
-    break;
-  }
-
- } else {
-  if ( strcmp(k, "mono") == 0 ) {
-   chans = 1;
-  } else if ( strcmp(k, "stereo") == 0 ) {
-   chans = 2;
-  } else {
-   chans = atoi(k);
-  }
-
-//  printf("mode: int; chans=%i, old_chans=%i\n", chans, old_chans);
-
-  if ( *cur + chans > max )
-   return -1;
-
-  for (i = 0; i < chans; i++) {
-   k   = arg[++(*cur)];
-   len = strlen(k);
-
-   mixer.mixer[i] = set_mixer_parse_volume(k, len, scale);
-  }
- }
-
  mixer.scale = scale;
 
- return roar_set_vol(con, id, &mixer, chans);
+ if ( roar_get_vol(con, id, &old_mixer, &old_chans) == -1 ) {
+  fprintf(stderr, "Error: can not get stream mixer info for stream %i\n", id);
+  return -1;
+ }
+
+ if ( strcmp(k, "mono") == 0 ) {
+  chans = 1;
+  mode  = ROAR_SET_VOL_UNMAPPED;
+ } else if ( strcmp(k, "stereo") == 0 ) {
+  chans = 2;
+  mode  = ROAR_SET_VOL_UNMAPPED;
+ } else {
+  chans = atoi(k);
+ }
+
+ // ensure most simple mode:
+ if ( old_chans == chans && mode == ROAR_SET_VOL_UNMAPPED )
+  mode = ROAR_SET_VOL_ALL;
+
+ if ( *cur + chans > max )
+  return -1;
+
+ for (i = 0; i < chans; i++) {
+  k   = arg[++(*cur)];
+  len = strlen(k);
+
+  mixer.mixer[i] = set_mixer_parse_volume(k, len, scale);
+ }
+
+ if ( roar_set_vol2(con, id, &mixer, chans, mode) == 0 )
+  return 0;
+
+ // no fallback if already using ROAR_SET_VOL_ALL:
+ if ( mode == ROAR_SET_VOL_ALL )
+  return -1;
+
+ if ( roar_conv_volume(&mixer, &mixer, old_chans, chans) == -1 )
+  return -1;
+
+ chans = old_chans;
+ mode  = ROAR_SET_VOL_ALL;
+
+ if ( roar_set_vol2(con, id, &mixer, chans, mode) == 0 )
+  return 0;
+
+ return -1;
 }
 
 
@@ -831,8 +1008,8 @@ int save_meta (struct roar_connection * con, int id, char * file) {
  return 0;
 }
 
-int set_flags (struct roar_connection * con, int id, int reset, char * flags) {
- int f = ROAR_FLAG_NONE;
+int set_flags (struct roar_connection * con, int id, int action, char * flags) {
+ uint32_t f = ROAR_FLAG_NONE;
  char * c;
  struct roar_stream s[1];
 
@@ -871,6 +1048,8 @@ int set_flags (struct roar_connection * con, int id, int reset, char * flags) {
    f |= ROAR_FLAG_IMMUTABLE;
   } else if ( !strcmp(c, "enhance") ) {
    f |= ROAR_FLAG_ENHANCE;
+  } else if ( !strcmp(c, "singlesink") ) {
+   f |= ROAR_FLAG_SINGLESINK;
   } else {
    fprintf(stderr, "Error: unknown flag: %s\n", c);
    return -1;
@@ -879,17 +1058,35 @@ int set_flags (struct roar_connection * con, int id, int reset, char * flags) {
   c = strtok(NULL, ",");
  }
 
- return roar_stream_set_flags(con, s, f, reset);
+ return roar_stream_set_flags2(con, s, f, action);
+}
+
+int set_role (struct roar_connection * con, int id, int role) {
+ struct roar_stream s;
+
+ if ( id == -1 )
+  return -1;
+
+ if ( roar_stream_new_by_id(&s, id) == -1 )
+  return -1;
+
+ if ( roar_stream_set_role(con, &s, role) == -1 )
+  return -1;
+
+ return 0;
 }
 
 int show_aiprofile (const char * profile) {
  struct roar_audio_info info;
+ const char * mime;
 
  if ( roar_profile2info(&info, profile) == -1 ) {
   fprintf(stderr, "Error: unknown profile: %s\n", profile);
   return -1;
  }
 
+ mime = roar_codec2mime(info.codec);
+
  printf("Profile Name          : %s\n", profile);
 
  if ( info.rate )
@@ -899,8 +1096,10 @@ int show_aiprofile (const char * profile) {
  if ( info.channels )
   printf("Profile channels      : %i\n", info.channels);
 
- printf("Profile codec         : %2i (%s%s)\n", info.codec, roar_codec2str(info.codec),
-                                       info.codec == ROAR_CODEC_DEFAULT ? " native" : "");
+ printf("Profile codec         : %2i (%s%s%s%s)\n", info.codec, roar_codec2str(info.codec),
+                                       info.codec == ROAR_CODEC_DEFAULT ? " native" : "",
+                                       mime == NULL ? "" : " mimetype:",
+                                       mime == NULL ? "" : mime);
 
  return 0;
 }
@@ -945,6 +1144,15 @@ int main (int argc, char * argv[]) {
   } else if ( strcmp(k, "--list-aiprofiles") == 0 ) {
    list_aiprofiles();
    return 0;
+  } else if ( strcmp(k, "--list-libstandards") == 0 ) {
+   lib_standards();
+   return 0;
+  } else if ( strcmp(k, "--enum-servers") == 0 ) {
+   enum_servers();
+   return 0;
+  } else if ( strcmp(k, "--hash-password") == 0 ) {
+   hash_password();
+   return 0;
   } else if ( *k == '-' ) {
    fprintf(stderr, "Error: unknown argument: %s\n", k);
    usage();
@@ -957,7 +1165,8 @@ int main (int argc, char * argv[]) {
  // connect
 
  if ( roar_simple_connect(&con, server, "roarctl") == -1 ) {
-  fprintf(stderr, "Error: Can not connect to server\n");
+  fprintf(stderr, "Error: Can not connect to server: %s: %s(%i)\n",
+          server == NULL ? "(default)" : server, roar_vs_strerr(roar_error), roar_error);
   return 1;
  }
 
@@ -976,7 +1185,7 @@ int main (int argc, char * argv[]) {
    usage();
 
   } else if ( !strcmp(k, "sleep") ) {
-   sleep(atoi(argv[++i]));
+   roar_sleep(atoi(argv[++i]));
 
   } else if ( !strcmp(k, "ping") ) {
 #ifdef ROAR_HAVE_GETTIMEOFDAY
@@ -1030,30 +1239,41 @@ int main (int argc, char * argv[]) {
 
   } else if ( !strcmp(k, "whoami") ) {
    printf("My client ID is: %i\n", roar_get_clientid(&con));
+  } else if ( !strcmp(k, "serverinfo") ) {
+   server_info(&con);
+  } else if ( !strcmp(k, "servertime") ) {
+   server_time(&con);
   } else if ( !strcmp(k, "serveroinfo") ) {
-   server_oinfo(&con);
+   server_oinfo(&con, -1);
+  } else if ( !strcmp(k, "serveroinfo2") ) {
+   t = roar_str2dir(argv[++i]);
+   if ( t == -1 ) {
+    fprintf(stderr, "Error: unknown stream direction: %s\n", argv[i]);
+   } else {
+    server_oinfo(&con, t);
+   }
+  } else if ( !strcmp(k, "serverstandards") ) {
+   server_standards(&con);
+  } else if ( !strcmp(k, "libstandards") ) {
+   lib_standards();
   } else if ( !strcmp(k, "listclients") ) {
    list_clients(&con);
+  } else if ( !strcmp(k, "clientinfo") ) {
+   display_client(&con, atoi(argv[++i]));
   } else if ( !strcmp(k, "liststreams") ) {
    list_streams(&con);
+  } else if ( !strcmp(k, "streaminfo") ) {
+   display_stream(&con, atoi(argv[++i]));
   } else if ( !strcmp(k, "allinfo") ) {
-   server_oinfo(&con);
+   server_oinfo(&con, -1);
    printf("\n");
    list_clients(&con);
    printf("\n");
    list_streams(&con);
 
   } else if ( !strcmp(k, "kick") ) {
-   k = argv[++i];
-   if ( !strcmp(k, "client") ) {
-    t = ROAR_OT_CLIENT;
-   } else if ( !strcmp(k, "stream") ) {
-    t = ROAR_OT_STREAM;
-   } else if ( !strcmp(k, "sample") ) {
-    t = ROAR_OT_SAMPLE;
-   } else if ( !strcmp(k, "source") ) {
-    t = ROAR_OT_SOURCE;
-   } else {
+   t = roar_str2ot((k = argv[++i]));
+   if ( t == -1 ) {
     fprintf(stderr, "Error: unknown type: %s\n", k);
     continue;
    }
@@ -1095,6 +1315,32 @@ int main (int argc, char * argv[]) {
     printf("flags changed\n");
    }
    i++;
+  } else if ( !strcmp(k, "toggleflag") ) {
+   i++;
+   if ( set_flags(&con, atoi(argv[i]), ROAR_TOGGLE_FLAG, argv[i+1]) == -1 ) {
+    fprintf(stderr, "Error: can not toggle flags\n");
+   } else {
+    printf("flags changed\n");
+   }
+   i++;
+  } else if ( !strcmp(k, "protectflag") ) {
+   i++;
+   if ( set_flags(&con, atoi(argv[i]), ROAR_NOOP_FLAG|ROAR_PROTECT_FLAG, argv[i+1]) == -1 ) {
+    fprintf(stderr, "Error: can not protect flags\n");
+   } else {
+    printf("flags protected\n");
+   }
+   i++;
+
+  } else if ( !strcmp(k, "role") ) {
+   i++;
+   if ( set_role(&con, atoi(argv[i]), roar_str2role(argv[i+1])) == -1 ) {
+    fprintf(stderr, "Error: can not set stream role\n");
+   } else {
+    printf("stream role set\n");
+   }
+   i++;
+
   } else if ( !strcmp(k, "metaset") ) {
    i++;
    if ( set_meta(&con, atoi(argv[i]), argv[i+1], argv[i+2], argv[i+3]) == -1 ) {
