@@ -1,7 +1,7 @@
 //roarphone.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2011
  *
  *  This file is part of roarclients a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -436,11 +436,11 @@ int main (int argc, char * argv[]) {
   } else if ( strcmp(k, "--io-flush") == 0 ) {
    g_conf.ioflush_interval = atoi(argv[++i]);
   } else if ( strcmp(k, "--rate") == 0 ) {
-   info.rate = atoi(argv[++i]);
+   info.rate = roar_str2rate(argv[++i]);
   } else if ( strcmp(k, "--bits") == 0 ) {
-   info.bits = atoi(argv[++i]);
+   info.bits = roar_str2bits(argv[++i]);
   } else if ( strcmp(k, "--channels") == 0 || strcmp(k, "--chans") == 0 ) {
-   info.channels = atoi(argv[++i]);
+   info.channels = roar_str2channels(argv[++i]);
 
   } else if ( strcmp(k, "--afi-downmix") == 0 ) {
    g_conf.filter.in.downmix = 1;
