@@ -1,7 +1,7 @@
 //driver_raw.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of roard a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -35,13 +35,13 @@ int driver_raw_open_vio(struct roar_vio_calls * inst, char * device, struct roar
  }
 
  if ( fh == -1 ) {
-  if ( (fh = open(device, O_CREAT|O_TRUNC|O_WRONLY, 0644)) == -1 ) {
+  if ( roar_vio_open_file(inst, device, O_CREAT|O_TRUNC|O_WRONLY, 0644) == -1 )
+   return -1;
+ } else {
+  if ( roar_vio_open_fh(inst, fh) == -1 )
    return -1;
-  }
  }
 
- roar_vio_set_fh(inst, fh);
-
  return 0;
 }
 #endif
