@@ -1,7 +1,7 @@
 //light.h:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of roard a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -32,17 +32,17 @@
 
 #define LIGHT_CHANNELS_DEFAULT 512 /* one DMX Universe */
 
-struct {
+extern struct light_state {
  unsigned int    channels;
  unsigned char * state;
  unsigned char * changes;
 } g_light_state;
 
-struct {
+extern struct light_mixer {
  int stream;
 } g_light_mixer;
 
-int light_init  (unsigned int channels);
+int light_init  (uint32_t channels);
 int light_free  (void);
 
 int light_update(void);
