@@ -1,7 +1,7 @@
 //meta.h:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of roard a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -29,9 +29,9 @@
 #include <roaraudio.h>
 
 #ifdef ROAR_SUPPORT_META
-int stream_meta_set     (int id, int type, char * name, char * val);
-int stream_meta_add     (int id, int type, char * name, char * val);
-int stream_meta_get     (int id, int type, char * name, char * val, size_t len);
+int stream_meta_set     (int id, int type, const char * name, const char * val);
+int stream_meta_add     (int id, int type, const char * name, const char * val);
+int stream_meta_get     (int id, int type, const char * name, char * val, size_t len);
 int stream_meta_list    (int id, int * types, size_t len);
 int stream_meta_clear   (int id);
 int stream_meta_finalize(int id);
