@@ -1,7 +1,7 @@
 //plugins.h:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2010-2011
  *
  *  This file is part of roard a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -42,6 +42,21 @@ int plugins_update   (void);
 int plugins_load     (const char * filename);
 
 int plugins_reg_sched(struct roard_plugins_sched * sched);
+int plugins_reg_proto(struct roard_proto         * proto);
+
+#define ROARD_DL_REG_PROTO(proto) \
+ static int __reg_proto(struct roar_dl_librarypara * para, struct roar_dl_libraryinst * lib) { \
+  size_t i; \
+ \
+  (void)para, (void)lib; \
+ \
+  for (i = 0; i < (sizeof((proto))/sizeof(*(proto))); i++) \
+   plugins_reg_proto(&((proto)[i])); \
+ \
+  return 0; \
+ }
+
+#define ROARD_DL_REGFN_PROTO() ROAR_DL_PLUGIN_REG(ROAR_DL_FN_PROTO, __reg_proto)
 
 #endif
 
