@@ -1,7 +1,7 @@
 //sample.h:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of roard a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -30,7 +30,7 @@
 
 #define ROAR_SAMPLES_MAX  32
 
-struct roar_sample * g_samples[ROAR_SAMPLES_MAX];
+extern struct roar_sample * g_samples[ROAR_SAMPLES_MAX];
 
 int samples_init      (void);
 int samples_free      (void);
