@@ -1,7 +1,7 @@
 //waveform.h:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2011
  *
  *  This file is part of roard a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -28,15 +28,18 @@
 
 #include <roaraudio.h>
 
-struct {
+struct waveform_stream {
  int stream;
  struct roar_stream_server * ss;
-} g_waveform_mixer;
+};
+
+extern struct waveform_stream g_waveform_mixer, g_waveform_recbridge;
 
 int waveform_init  (void);
 int waveform_free  (void);
 
-int waveform_update_mixer (void);
+int waveform_update_inputs (void);
+int waveform_update_mixer  (void);
 
 #endif
 
