@@ -1,7 +1,7 @@
 //midi.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2010
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2011
  *
  *  This file is part of roard a part of RoarAudio,
  *  a cross-platform sound system for both, home and professional use.
@@ -27,6 +27,16 @@
 
 #ifndef ROAR_WITHOUT_DCOMP_MIDI
 
+// declared 'extern'
+struct midi_config midi_config;
+struct midi_state_mess g_midi_mess;
+#ifndef ROAR_WITHOUT_DCOMP_CB
+struct midi_state_cb g_midi_cb;
+#endif
+struct midi_clock g_midi_clock;
+struct midi_mixer g_midi_mixer;
+// //
+
 #if defined(ROAR_HAVE_IO_POSIX) && !defined(ROAR_TARGET_WIN32)
 #define _HAVE_CONSOLE
 #endif
@@ -135,7 +145,7 @@ int midi_check_stream  (int id) {
  struct roar_stream        *   s;
  struct roar_stream_server *  ss;
  struct roar_buffer        *   b;
- char                      * buf;
+ void                      * buf;
  ssize_t                     len;
 
  if ( g_streams[id] == NULL )
@@ -156,9 +166,9 @@ int midi_check_stream  (int id) {
     return -1;
  }
 
- if ( roar_buffer_new_data(&b, MIDI_READ_SIZE, (void**)&buf) == -1 ) {
-  ROAR_ERR("midi_check_stream(*): Can not alloc buffer space!");
-  ROAR_DBG("midi_check_stream(*) = -1");
+ if ( roar_buffer_new_data(&b, MIDI_READ_SIZE, &buf) == -1 ) {
+  ROAR_ERR("midi_check_stream(id=%i): Can not alloc buffer space!", id);
+  ROAR_DBG("midi_check_stream(id=%i) = -1", id);
   return -1;
  }
 
@@ -168,7 +178,12 @@ int midi_check_stream  (int id) {
   return -1;
  }
 
- roar_buffer_set_len(b, len);
+ if ( roar_buffer_set_len(b, len) == -1 ) {
+  ROAR_WARN("midi_check_stream(id=%i): Can not set buffer size. BAD.", id);
+  roar_buffer_free(b);
+  streams_delete(id);
+  return -1;
+ }
 
  if ( stream_add_buffer(id, b) == -1 ) {
   roar_buffer_free(b);
@@ -185,6 +200,7 @@ int midi_check_stream  (int id) {
     return -1;
  }
 
+ ROAR_DBG("midi_check_stream(id=%i) = 0", id);
  return 0;
 }
 
@@ -211,7 +227,176 @@ int midi_send_stream   (int id) {
  return 0;
 }
 
+#if 1
+static int _cache_to_mes(struct midi_message * mes, unsigned char * data, size_t len,
+                         int id, struct roar_stream_server * ss) {
+ unsigned char type = *data;
+
+ ROAR_DBG("_cache_to_mes(*): type=0x%.2X", (unsigned int)type);
+
+ if (type == MIDI_TYPE_CLOCK_TICK || type == MIDI_TYPE_CLOCK_START || type == MIDI_TYPE_CLOCK_STOP ) {
+  if ( len != 1 )
+   return -1;
+
+  mes->type = type;
+
+  if ( type == MIDI_TYPE_CLOCK_TICK ) {
+// TODO:
+   ROAR_STREAM(ss)->pos = ROAR_MATH_OVERFLOW_ADD(ROAR_STREAM(ss)->pos, 1);
+  }
+ } else {
+  mes->type    = type & 0xF0;
+  mes->channel = type & 0x0F;
+
+  switch (mes->type) {
+   case MIDI_TYPE_NOTE_ON:
+   case MIDI_TYPE_NOTE_OFF:
+     if ( len != 3 )
+      return -1;
+     roar_midi_note_from_midiid(&(mes->d.note), data[1]);
+
+     // if velocity is zero we have a NOTE OFF event
+     if ( data[2] == 0 )
+      mes->type = MIDI_TYPE_NOTE_OFF;
+   case MIDI_TYPE_PA:
+   case MIDI_TYPE_CONTROLER:
+     mes->kk = data[1];
+     mes->vv = data[2];
+    break;
+   case MIDI_TYPE_PROGRAM:
+   case MIDI_TYPE_MA:
+     if ( len != 2 )
+      return -1;
+     mes->vv = data[1];
+    break;
+   case MIDI_TYPE_PB:
+   case MIDI_TYPE_SYSEX:
+     ROAR_WARN("_cache_to_mes(id=%i): Message of Type 0x%.2X (PB or SysEx) not yet supported", id, (unsigned int)type);
+     return -1;
+    break;
+  }
+ }
+
+ switch (mes->type) {
+  case MIDI_TYPE_CONTROLER:
+    switch (mes->kk) {
+     case MIDI_CCE_MAIN_VOL:
+       if ( (516U * (unsigned int)mes->vv) > 65100 ) { // max volume
+        ss->mixer.mixer[mes->channel] = 65535;
+       } else {
+        ss->mixer.mixer[mes->channel] = 516 * mes->vv;
+       }
+      break;
+    }
+   break;
+ }
+
+ ROAR_DBG("_cache_to_mes(*) = 0");
+ return 0;
+}
+
 int midi_conv_midi2mes (int id) {
+ struct roar_stream        *   s;
+ struct roar_stream_server *  ss;
+ size_t have;
+ void * bufdata;
+ unsigned char * data;
+ unsigned char   cache[3];
+ size_t cachepos = 0;
+ int sync = 1;
+ struct roar_buffer        * buf = NULL;
+ struct midi_message       * mes = NULL;
+
+ if ( g_streams[id] == NULL )
+  return -1;
+
+ ROAR_DBG("midi_conv_midi2mes(id=%i) = ?", id);
+
+ s = ROAR_STREAM(ss = g_streams[id]);
+
+ while (ss->buffer != NULL) {
+  if ( roar_buffer_get_len(ss->buffer, &have) == -1 )
+   return -1;
+  if ( roar_buffer_get_data(ss->buffer, &bufdata) == -1 )
+   return -1;
+  data = bufdata;
+
+  while (have) {
+   if ( !sync ) {
+    if ( *data & 0x80 ) {
+     cachepos = 0;
+     sync = 1;
+    } else {
+     data++;
+     have--;
+     continue;
+    }
+   }
+
+   if ( (*data & 0x80) && cachepos != 0 ) {
+    // submit cache.
+
+    if ( midi_new_bufmes(&buf, &mes) == -1 ) {
+     // this is bad.
+     // TODO: handle better.
+     return -1;
+    }
+
+    if ( _cache_to_mes(mes, cache, cachepos, id, ss) == -1 ) {
+     ROAR_WARN("midi_conv_midi2mes(id=%i): Message can not be parsed.", id);
+     roar_buffer_free(buf);
+    } else {
+     midi_add_buf(id, buf); // FIXME: don't we need error handling here?
+    }
+
+    cachepos = 0;
+   } else if ( cachepos == sizeof(cache) ) {
+    // lost sync.
+    sync = 0;
+    continue;
+   }
+
+   ROAR_DBG("midi_conv_midi2mes(id=%i): cache[%u] = 0x%.2X", id, (unsigned int)cachepos, (unsigned int)*data);
+   cache[cachepos++] = *data;
+   data++;
+   have--;
+  }
+
+  if ( roar_buffer_next(&(ss->buffer)) == -1 ) {
+   ROAR_WARN("midi_conv_midi2mes(id=%i): Can not move to next buffer segment. BAD.", id);
+   break;
+  }
+ }
+
+ if ( cachepos ) {
+  if ( midi_new_bufmes(&buf, &mes) == -1 ) {
+   // this is bad.
+   // TODO: handle better.
+   return -1;
+  }
+
+  if ( _cache_to_mes(mes, cache, cachepos, id, ss) == -1 ) {
+   roar_buffer_free(buf);
+  } else {
+   midi_add_buf(id, buf); // FIXME: don't we need error handling here?
+   cachepos = 0;
+  }
+ }
+
+ if ( cachepos ) {
+  if ( roar_buffer_new_data(&buf, cachepos, &bufdata) == -1 )
+   return -1;
+
+  memcpy(bufdata, cache, cachepos);
+
+  ss->buffer = buf;
+ }
+
+ return 0;
+}
+#else
+int midi_conv_midi2mes (int id) {
+ void          * bufdata;
  unsigned char * data;
  struct roar_stream        *   s;
  struct roar_stream_server *  ss;
@@ -228,11 +413,15 @@ int midi_conv_midi2mes (int id) {
  s = ROAR_STREAM(ss = g_streams[id]);
 
  while (ss->buffer != NULL && alive) {
-  if ( roar_buffer_get_data(ss->buffer, (void**)&data) == -1 ) {
+  ROAR_DBG("midi_conv_midi2mes(id=%i): ss->buffer=%p, alive=%i", id, ss->buffer, alive);
+
+  if ( roar_buffer_get_data(ss->buffer, &bufdata) == -1 ) {
    alive = 0;
    continue;
   }
 
+  data = bufdata;
+
   if ( roar_buffer_get_len(ss->buffer, &have) == -1 ) {
    alive = 0;
    continue;
@@ -240,6 +429,8 @@ int midi_conv_midi2mes (int id) {
   old_have = have;
 
   while (have && alive) {
+   ROAR_DBG("midi_conv_midi2mes(id=%i): have=%llu, alive=%i", id, (long long unsigned int)have, alive);
+
    if ( *data & 0x80 ) {
     if ( buf != NULL ) {
      midi_add_buf(id, buf);
@@ -349,6 +540,7 @@ int midi_conv_midi2mes (int id) {
   }
 
   if ( need ) {
+   ROAR_DBG("midi_conv_midi2mes(id=%i) = ?", id);
    if ( roar_buffer_set_offset(ss->buffer, old_have - last_have) == -1 ) {
     ROAR_ERR("midi_conv_midi2mes(*): FIXME: BUG!!! Need to restore buffer here with corrected length");
    }
@@ -367,6 +559,7 @@ int midi_conv_midi2mes (int id) {
 
  return 0;
 }
+#endif
 
 #define _nb  len++; *(d++)
 int midi_conv_mes2midi (int id) {
@@ -374,6 +567,7 @@ int midi_conv_mes2midi (int id) {
  struct roar_stream_server *  ss;
  struct roar_buffer        * buf = g_midi_mess.buf;
  struct midi_message       * mes = NULL;
+ void                      * bufdata;
  unsigned char               data[3];
  unsigned char             * d;
  int                         len;
@@ -389,10 +583,12 @@ int midi_conv_mes2midi (int id) {
  send_clock = streams_get_flag(id, ROAR_FLAG_SYNC);
 
  while (buf != NULL) {
-  if ( roar_buffer_get_data(buf, (void**)&mes) == -1 ) {
+  if ( roar_buffer_get_data(buf, &bufdata) == -1 ) {
    return -1;
   }
 
+  mes = bufdata;
+
   if (mes->type == MIDI_TYPE_CLOCK_TICK || mes->type == MIDI_TYPE_CLOCK_START || mes->type == MIDI_TYPE_CLOCK_STOP ) {
    if ( send_clock ) {
     if ( stream_vio_s_write(ss, &(mes->type), 1) != 1 ) {
@@ -414,7 +610,7 @@ int midi_conv_mes2midi (int id) {
     case MIDI_TYPE_CONTROLER:
       switch (mes->kk) {
        case MIDI_CCE_MAIN_VOL:
-         if ( 516 * mes->vv > 65100 ) { // max volume
+         if ( 516U * (unsigned int)mes->vv > 65100 ) { // max volume
           ss->mixer.mixer[mes->channel] = 65535;
          } else {
           ss->mixer.mixer[mes->channel] = 516 * mes->vv;
@@ -454,12 +650,15 @@ int midi_conv_mes2midi (int id) {
 int midi_conv_mes2ssynth(void) {
  struct roar_buffer        * buf = g_midi_mess.buf;
  struct midi_message       * mes = NULL;
+ void                      * bufdata;
 
  while (buf != NULL) {
-  if ( roar_buffer_get_data(buf, (void**)&mes) == -1 ) {
+  if ( roar_buffer_get_data(buf, &bufdata) == -1 ) {
    return -1;
   }
 
+  mes = bufdata;
+
   if ( ssynth_eval_message(mes) == -1 )
    return -1;
 
@@ -490,26 +689,37 @@ int midi_new_bufmes    (struct roar_buffer ** buf, struct midi_message ** mes) {
 }
 
 int midi_add_buf       (int id, struct roar_buffer * buf) {
+#ifdef DEBUG
  struct midi_message * mes;
+ void * bufdata;
+#endif
 
  if ( id == -1 || buf == NULL )
   return -1;
 
- if ( roar_buffer_get_data(buf, (void**)&mes) == -1 )
+#ifdef DEBUG
+ if ( roar_buffer_get_data(buf, &bufdata) == -1 )
   return -1;
 
+ mes = bufdata;
+
  ROAR_DBG("midi_add_buf(id=%i, buf=%p) = ?", id, buf);
  ROAR_DBG("midi_add_buf(*): MIDI Message of Type 0x%.2X", mes->type);
  ROAR_DBG("midi_add_buf(*): Channel: %i", mes->channel);
  ROAR_DBG("midi_add_buf(*): flags=0x%.2X", mes->flags);
  ROAR_DBG("midi_add_buf(*): kk=0x%.2X, vv=0x%.2X", mes->kk, mes->vv);
+#endif
 
  if ( g_midi_mess.buf == NULL ) {
   g_midi_mess.buf = buf;
  } else {
-  roar_buffer_add(g_midi_mess.buf, buf);
+  if ( roar_buffer_add(g_midi_mess.buf, buf) == -1 ) {
+   ROAR_DBG("midi_add_buf(*) = -1");
+   return -1;
+  }
  }
 
+ ROAR_DBG("midi_add_buf(*) = 0");
  return 0;
 }
 
@@ -802,16 +1012,19 @@ int midi_cb_stop (void) {
 int midi_cb_readbuf(void) {
  struct roar_buffer        * buf = g_midi_mess.buf;
  struct midi_message       * mes = NULL;
+ void                      * bufdata;
 
  ROAR_DBG("midi_cb_readbuf(void) = ?");
 
  while (buf != NULL) {
   ROAR_DBG("midi_cb_readbuf(void): buf=%p", buf);
 
-  if ( roar_buffer_get_data(buf, (void**)&mes) == -1 ) {
+  if ( roar_buffer_get_data(buf, &bufdata) == -1 ) {
    return -1;
   }
 
+  mes = bufdata;
+
   switch (mes->type) {
    case MIDI_TYPE_NOTE_ON:
      midi_cb_start(mes->d.note.freq);
